from typing import Any, ClassVar, Dict

AddressLookupOptions = Dict[str, Any]


class AddressLookup:
    code: ClassVar[str]
    options: AddressLookupOptions
    assets: ClassVar[Dict[str, Any]] = None

    def __init__(self, options: AddressLookupOptions):
        self.options = options

    def get_frontend_options(self) -> AddressLookupOptions:
        return {}

    def get_assets(self):
        return self.assets or {}



from rest_framework import serializers

from wagtaildonate.exceptions import PaymentMethodNotFound
from wagtaildonate.payment_methods.base import PaymentMethod
from wagtaildonate.payment_methods.utils import get_payment_method


class PaymentMethodField(serializers.Field):
    """
    Retrieve payment method based on its code.
    """

    def __init__(self, *args, **kwargs):
        self.max_length = int(kwargs.pop("max_length", 255))
        super().__init__(*args, **kwargs)

    def to_internal_value(self, data: str) -> PaymentMethod:
        if not isinstance(data, str):
            msg = "Incorrect type. Expected a string, but got %s."
            raise serializers.ValidationError(msg % type(data).__name__)

        if len(data) > self.max_length:
            raise serializers.ValidationError("String is too long.")
        try:
            payment_method_context = self.context.get("payment_method_context")
            payment_method = get_payment_method(data, payment_method_context)
        except PaymentMethodNotFound as e:
            raise serializers.ValidationError("Payment method not configured") from e

        return payment_method

    def to_representation(self, value: PaymentMethod) -> str:
        return value.code


class PaymentMethodAndFrequencySerializer(serializers.Serializer):
    """
    Validate payment method and frequency.
    """

    payment_method = PaymentMethodField()
    frequency = serializers.CharField(max_length=255)

    def validate(self, data):
        if not data["payment_method"].is_frequency_supported(data["frequency"]):
            raise serializers.ValidationError(
                {"frequency": ["The frequency is not supported by the payment method."]}
            )
        return data



class TransactionStatus:
    provider_transaction_status: str
    transaction_status: str

    def __init__(self, *, provider_transaction_status: str, transaction_status: str):
        self.provider_transaction_status = provider_transaction_status
        self.transaction_status = transaction_status



from bs4 import BeautifulSoup, NavigableString


# Taken from (Sorry. flake8 made me chop it in half!)
# https://github.com/wagtail/wagtail-localize/blob/316757bb80bf0370a36eb4efad18
# ca47905102a2/wagtail_localize/translation/segments/html.py#L230-L263
def extract_html_elements(soup):
    """
    Extracts HTML elements from a fragment. Returns the plain text representation
    of the HTML document and an array of elements including their span, type and attributes.
    For example:
    text, elements = extract_html_elements("This is a paragraph. <b>This is some bold <i>and now italic</i></b> text")
    text == "This is a paragraph. This is some bold and now italic text"
    elements == [(39, 53, 'i', {}), (21, 53, 'b', {})]
    """
    texts = []
    cursor = {"current": 0}
    elements = []

    def walk(soup):
        for element in soup.children:
            if isinstance(element, NavigableString):
                texts.append(element)
                cursor["current"] += len(element)

            else:
                start = cursor["current"]
                walk(element)
                end = cursor["current"]

                elements.append((start, end, element.name, element.attrs.copy()))

    walk(soup)

    return "".join(texts), elements


def rich_text_to_plain_text(value):
    """
    Converts rich text field values into plain text.

    All HTML tags are removed, blocks are separated by new lines and list items are prefixed.

    Doesn't yet handle headings, links or inline formatting.
    """
    soup = BeautifulSoup(value, "html.parser")
    lines = []

    def process_block_tag(tag, newline_prefix=""):
        # TODO: Extract link URLs
        text, elements = extract_html_elements(tag)

        # Insert line breaks
        line_breaks = []
        for start, end, tag_name, attrs in elements:
            if tag_name == "br":
                line_breaks.append(start)

        if line_breaks:
            new_text = ""
            cursor = 0

            for pos in line_breaks:
                # Insert text up until the line break
                if pos > cursor:
                    new_text += text[cursor:pos]
                    cursor = pos

                # Insert line break
                new_text += "\n"

                # Insert newline prefix
                new_text += newline_prefix

            # Insert text after last line break
            if cursor < len(text):
                # Black and flake8 disagree on how to format this line
                new_text += text[cursor : len(text)]  # noqa

            text = new_text

        return text

    def process_paragraph(tag):
        lines.append(process_block_tag(tag))
        lines.append("")

    def process_unordered_list(tag):
        for list_item in tag:
            lines.append(" * " + process_block_tag(list_item, newline_prefix="   "))

        lines.append("")

    def process_ordered_list(tag):
        for i, list_item in enumerate(tag):
            prefix = "{}. ".format(i + 1)
            lines.append(
                prefix + process_block_tag(list_item, newline_prefix=" " * len(prefix))
            )

        lines.append("")

    def walk(tag):
        if tag.name == "div":
            # Step into div tags
            for child in tag:
                walk(child)

        elif tag.name in ["p", "h1", "h2", "h3", "h4", "h5", "h6"]:
            process_paragraph(tag)

        elif tag.name == "ul":
            process_unordered_list(tag)

        elif tag.name == "ol":
            process_ordered_list(tag)

        else:
            # Ignore everything else
            return

    for tag in soup:
        walk(tag)

    return "\n".join(lines).strip("\n")



from django import forms
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from wagtail.admin.forms import WagtailAdminPageForm
from wagtail.admin.widgets import AdminDateInput


class DonationAdminForm(WagtailAdminPageForm):
    def clean(self):
        cleaned_data = super().clean()

        # At least one of monthly or single is enabled
        if (
            not cleaned_data["allow_single_donations"]
            and not cleaned_data["allow_regular_donations"]
        ):
            self.add_error(
                "allow_single_donations",
                _("Please allow at least one donation option."),
            )

        # If 'show other' amount field is not true, donation values must be added
        # according to allowed frequency regular/single
        if not cleaned_data.get("show_other_amount_option"):
            single_amount_forms = self.formsets.get("suggested_amounts_single")
            if (
                single_amount_forms
                and len(single_amount_forms) < 1
                and cleaned_data.get("allow_single_donations")
            ):
                self.add_error(
                    "allow_single_donations", _("Please add donation values.")
                )

            regular_amount_forms = self.formsets.get("suggested_amounts_regular")
            if (
                regular_amount_forms
                and len(regular_amount_forms) < 1
                and cleaned_data.get("allow_regular_donations")
            ):
                self.add_error(
                    "allow_regular_donations", _("Please add donation values.")
                )

        return cleaned_data


def today():
    return timezone.now().date()


class DateRangeForm(forms.Form):

    from_date = forms.DateField(widget=AdminDateInput, initial=today)
    to_date = forms.DateField(widget=AdminDateInput, initial=today)

    def clean_from_date(self):
        from_date = self.cleaned_data["from_date"]
        if from_date > today():
            raise forms.ValidationError(_("Date cannot be in the future"))
        return from_date

    def clean_to_date(self):
        to_date = self.cleaned_data["to_date"]
        if to_date > today():
            raise forms.ValidationError(_("Date cannot be in the future"))
        return to_date

    def clean(self):
        from_date = self.cleaned_data.get("from_date")
        to_date = self.cleaned_data.get("to_date")
        if to_date and from_date and to_date < from_date:
            # swap them around
            self.cleaned_data["from_date"] = to_date
            self.cleaned_data["to_date"] = from_date



from django.test import TestCase, override_settings
from django.urls import reverse

from wagtail.models import Page, Site

from wagtaildonate.factories import (
    DonationFundraisingPayInPageFactory,
    DonationPageFactory,
)
from wagtaildonate.models import PayInEvent


class TestDonationConfigurationAPIView(TestCase):
    def setUp(self):
        root_page = Page.objects.get(id=1)
        Site.objects.create(hostname="test", port=80, root_page=root_page)
        self.donation_page = DonationPageFactory()
        self.pay_in_page = DonationFundraisingPayInPageFactory()
        PayInEvent.objects.create(
            event_code="evt1",
            event_name="Event 1",
            fundraiser_reference_required=True,
        )

    def test_pay_in_page_detail_in_response(self):
        response = self.client.get(reverse("wagtaildonate:api:configuration-donations"))
        self.assertEqual(response.status_code, 200)
        configuration_data = response.json()
        self.assertEqual(
            configuration_data["pay_in_page_id"],
            self.pay_in_page.id,
        )
        self.assertEqual(
            configuration_data["pay_in_success_url"],
            self.pay_in_page.get_thank_you_page_url(),
        )

    @override_settings(WAGTAIL_DONATE={"PAY_IN_EVENTS_ENABLED": False})
    def test_no_pay_in_events_when_disabled(self):
        response = self.client.get(reverse("wagtaildonate:api:configuration-donations"))
        self.assertEqual(response.status_code, 200)
        configuration_data = response.json()
        self.assertEqual(len(configuration_data["pay_in_events"]), 0)

    def test_correct_pay_in_events_other_event_enabled(self):
        response = self.client.get(reverse("wagtaildonate:api:configuration-donations"))
        self.assertEqual(response.status_code, 200)
        configuration_data = response.json()
        self.assertEqual(len(configuration_data["pay_in_events"]), 2)
        self.assertEqual(configuration_data["pay_in_events"][0]["event_code"], "evt1")
        self.assertEqual(configuration_data["pay_in_events"][1]["event_code"], "other")

    @override_settings(WAGTAIL_DONATE={"PAY_IN_OTHER_EVENT_ENABLED": False})
    def test_correct_pay_in_events_other_event_disabled(self):
        response = self.client.get(reverse("wagtaildonate:api:configuration-donations"))
        self.assertEqual(response.status_code, 200)
        configuration_data = response.json()
        self.assertEqual(len(configuration_data["pay_in_events"]), 1)
        self.assertEqual(configuration_data["pay_in_events"][0]["event_code"], "evt1")

    def test_correct_other_event_fundraiser_reference_required_enabled(self):
        response = self.client.get(reverse("wagtaildonate:api:configuration-donations"))
        self.assertEqual(response.status_code, 200)
        configuration_data = response.json()
        self.assertEqual(len(configuration_data["pay_in_events"]), 2)
        self.assertEqual(configuration_data["pay_in_events"][1]["event_code"], "other")
        self.assertTrue(
            configuration_data["pay_in_events"][1]["fundraiser_reference_required"]
        )

    @override_settings(
        WAGTAIL_DONATE={"PAY_IN_OTHER_EVENT_REQUIRE_FUNDRAISER_REFERENCE": False}
    )
    def test_correct_other_event_fundraiser_reference_required_disabled(self):
        response = self.client.get(reverse("wagtaildonate:api:configuration-donations"))
        self.assertEqual(response.status_code, 200)
        configuration_data = response.json()
        self.assertEqual(len(configuration_data["pay_in_events"]), 2)
        self.assertEqual(configuration_data["pay_in_events"][1]["event_code"], "other")
        self.assertFalse(
            configuration_data["pay_in_events"][1]["fundraiser_reference_required"]
        )



import logging

from django.utils.translation import gettext_lazy as _

import gocardless_pro
from rest_framework import serializers

from wagtaildonate.api.serializers.utils import get_recurring_donation_serializer_class
from wagtaildonate.models.utils import get_recurring_donation_model

logger = logging.getLogger(__name__)


RecurringDonationSerializer = get_recurring_donation_serializer_class()


class GoCardlessDonationSerializer(RecurringDonationSerializer):  # noqa
    account_holder_name = serializers.CharField(required=True, write_only=True)
    sort_code = serializers.RegexField(r"^\d{6}$", required=True, write_only=True)
    account_number = serializers.RegexField(
        r"^\d{6,8}$", required=True, write_only=True
    )
    account_holder_confirmation = serializers.BooleanField(
        default=False, write_only=True
    )

    class Meta(RecurringDonationSerializer.Meta):
        model = get_recurring_donation_model()
        fields = RecurringDonationSerializer.Meta.fields + [
            "account_holder_name",
            "sort_code",
            "account_number",
            "account_holder_confirmation",
        ]

    def validate_account_holder_confirmation(self, value):
        if not value:
            raise serializers.ValidationError(
                _(
                    "You must confirm that you are authorised to set up "
                    "Direct Debit on this account."
                )
            )
        return value

    def create(self, validated_data):
        # FIXME: Prevent double-submission

        client = self.context["payment_method"].get_client()

        # Create customer
        try:
            customer = client.customers.create(
                params={
                    "given_name": validated_data["first_name"],
                    "family_name": validated_data["surname"],
                    "address_line1": validated_data["address_line_1"],
                    "address_line2": validated_data["address_line_2"],
                    "city": validated_data["town"],
                    "postal_code": validated_data["postal_code"],
                    "country_code": validated_data["country"],
                    "email": validated_data["email"],
                }
            )
        except gocardless_pro.errors.ValidationFailedError as e:
            self.handle_gocardless_exception(e)

        # Create bank account
        try:
            bank_account = client.customer_bank_accounts.create(
                params={
                    "account_holder_name": validated_data["account_holder_name"],
                    "branch_code": validated_data["sort_code"],
                    "account_number": validated_data["account_number"],
                    "country_code": "GB",
                    "currency": "GBP",
                    "links": {"customer": customer.id},
                }
            )
        except gocardless_pro.errors.ValidationFailedError as e:
            self.handle_gocardless_exception(e)

        try:
            # Create Direct Debit mandate
            mandate = client.mandates.create(
                params={"links": {"customer_bank_account": bank_account.id}}
            )

            # Create subscription
            subscription = client.subscriptions.create(
                params={
                    "interval_unit": "monthly",
                    "amount": int(validated_data["amount"] * 100),
                    "currency": "GBP",
                    "links": {"mandate": mandate.id},
                }
            )
        except gocardless_pro.errors.ValidationFailedError as e:
            self.handle_gocardless_exception(e)

        validated_data["subscription_id"] = subscription.id

        # Delete non-database fields.
        del validated_data["account_holder_name"]
        del validated_data["sort_code"]
        del validated_data["account_number"]
        del validated_data["account_holder_confirmation"]

        return super().create(validated_data)

    def handle_gocardless_exception(self, exception):
        logger.info("GoCardless transaction unsuccessful: %r", exception)
        if not isinstance(
            exception,
            (
                gocardless_pro.errors.ValidationFailedError,
                gocardless_pro.errors.InvalidStateError,
            ),
        ):
            logger.exception(
                "Unexpected exception raised by GoCardless while setting up a new Direct Debit"
            )
            raise serializers.ValidationError(
                {
                    "direct_debit_errors": [
                        _(
                            "There was an error with setting up your Direct Debit payment. Please try again later."
                        )
                    ]
                }
            )
        error_dict = {}
        account_number_errors = list(
            self.filter_account_number_errors(exception.errors)
        )
        sort_code_errors = list(self.filter_sort_code_errors(exception.errors))
        account_holder_name_errors = list(
            self.filter_account_holder_name_errors(exception.errors)
        )
        postal_code_errors = list(self.filter_postal_code_errors(exception.errors))
        if account_number_errors:
            error_dict["account_number"] = account_number_errors
        if sort_code_errors:
            error_dict["sort_code"] = sort_code_errors
        if account_holder_name_errors:
            error_dict["account_holder_name"] = account_holder_name_errors
        if postal_code_errors:
            error_dict["postal_code"] = postal_code_errors
        if not any(error_dict.values()):
            error_dict["direct_debit_errors"] = [
                _(
                    "Sorry. There was an error processing your Direct Debit payment. "
                    "Plase try again or contact us."
                )
            ]
        raise serializers.ValidationError(error_dict)

    def filter_account_number_errors(self, errors):
        for error in errors:
            if error.get("field") != "account_number":
                continue
            message = error.get("message")
            if message:
                yield f"Account number {message}"

    def filter_sort_code_errors(self, errors):
        for error in errors:
            if error.get("field") != "branch_code":
                continue
            message = error.get("message")
            if message:
                yield f"Sort code {message}"

    def filter_account_holder_name_errors(self, errors):
        for error in errors:
            if error.get("field") != "account_holder_name":
                continue
            message = error.get("message")
            if message:
                yield f"Account holder name {message}"

    def filter_postal_code_errors(self, errors):
        for error in errors:
            if error.get("field") != "postal_code":
                continue
            message = error.get("message")
            if message:
                yield f"Postal code {message}"



from typing import Generator

from django.utils.module_loading import import_string

from wagtaildonate.exceptions import PaymentMethodNotFound
from wagtaildonate.payment_methods.base import PaymentMethod
from wagtaildonate.settings import donate_settings


def instantiate_payment_method(setting, klass=None, context=None) -> PaymentMethod:
    if klass is None:
        klass = import_string(setting["class"])
    return klass(setting.get("options", {}), context=context)


def get_payment_method(user_payment_method, context=None) -> PaymentMethod:
    for payment_method in donate_settings.PAYMENT_METHODS:
        klass = import_string(payment_method["class"])
        if klass.code == user_payment_method:
            return instantiate_payment_method(
                payment_method, klass=klass, context=context
            )
    raise PaymentMethodNotFound(user_payment_method)


def get_all_payment_methods(
    *, allowed_frequencies=None, payment_method_context=None
) -> Generator[PaymentMethod, None, None]:
    for payment_method in donate_settings.PAYMENT_METHODS:
        payment_method_object = instantiate_payment_method(
            payment_method, context=payment_method_context
        )
        if allowed_frequencies and not any(
            payment_method_object.is_frequency_supported(f) for f in allowed_frequencies
        ):
            continue
        yield payment_method_object



import django

DEFAULTS = {
    # Payment methods list
    "PAYMENT_METHODS": [],
    # Contact details
    "PHONE_NUMBER_REQUIRED": True,
    "EMAIL_REQUIRED": True,
    # Base donation model and serializer
    "SINGLE_DONATION_MODEL": "wagtaildonate.SingleDonation",
    "SINGLE_DONATION_SERIALIZER_CLASS": "wagtaildonate.api.serializers.donations.SingleDonationSerializer",
    "RECURRING_DONATION_MODEL": "wagtaildonate.RecurringDonation",
    "RECURRING_DONATION_SERIALIZER_CLASS": "wagtaildonate.api.serializers.donations.RecurringDonationSerializer",
    "PAY_IN_MODEL": "wagtaildonate.PayIn",
    "PAY_IN_SERIALIZER_CLASS": "wagtaildonate.api.serializers.donations.PayInSerializer",
    "PAY_IN_EVENT_MODEL": "wagtaildonate.PayInEvent",
    "PAY_IN_EVENT_SERIALIZER_CLASS": "wagtaildonate.api.serializers.configuration.PayInEventSerializer",
    # Configuration endpoint
    "CONFIGURATION_CLASS": "wagtaildonate.configuration.Configuration",
    "CONFIGURATION_SERIALIZER_CLASS": "wagtaildonate.api.serializers.configuration.ConfigurationSerializer",
    # Wagtail Admin settings
    "DONATION_EXPORT_ENABLED": True,
    "DONATION_EXPORT_LOG_ENABLED": True,
    "DONATION_EXPORT_LOG_MODEL": "wagtaildonate.DonationExportLog",
    "THREE_D_SECURE_EXPORT_FIELDS": False,
    # Default page settings
    "GET_DEFAULT_DONATION_PAGE_INSTANCE_CALLABLE": None,
    # Address lookup
    "ADDRESS_LOOKUP": {
        "class": "wagtaildonate.address_lookups.generic.GenericAddressLookup",
    },
    # Minimum and maximum amount
    "MINIMUM_AMOUNT_PER_FREQUENCY": {
        "single": "1.00",
        "monthly": "1.00",
        "payin": "1.00",
    },
    "MAXIMUM_AMOUNT_PER_FREQUENCY": {
        "single": "10000",
        "monthly": "10000",
        "payin": "10000",
    },
    "PAY_IN_EVENTS_ENABLED": True,
    "PAY_IN_OTHER_EVENT_ENABLED": True,
    "PAY_IN_OTHER_EVENT_REQUIRE_FUNDRAISER_REFERENCE": True,
    # reCAPTCHA
    "RECAPTCHA_PUBLIC_KEY": "",
    "RECAPTCHA_PRIVATE_KEY": "",
    "RECAPTCHA_MINIMUM_SCORE": 0.5,
}


class WagtailDonateSettings:
    def __getattr__(self, attr):
        django_settings = getattr(django.conf.settings, "WAGTAIL_DONATE", {})

        try:
            # Check if present in user settings
            return django_settings[attr]
        except KeyError:
            return DEFAULTS[attr]


donate_settings = WagtailDonateSettings()



import string
from decimal import Decimal

from wagtail.models import Page

import factory
import factory.django
import factory.fuzzy

from wagtaildonate.models import SingleDonation
from wagtaildonate.tests.testapp.models import (
    TestDonationCampaignPage,
    TestDonationFundraisingPayInPage,
    TestDonationPage,
)


class DonationPageFactory(factory.django.DjangoModelFactory):
    title = factory.Sequence(lambda n: "donation_page_%d" % n)
    slug = factory.Sequence(lambda n: "donation_page_%d" % n)
    thank_you_email_subject = ""
    thank_you_email_content = ""

    class Meta:
        model = TestDonationPage

    @classmethod
    def _create(cls, model_class, *args, **kwargs):
        root_page = Page.objects.all().order_by("depth").first()
        donation_page = model_class(*args, **kwargs)
        root_page.add_child(instance=donation_page)
        donation_page.save()
        return donation_page


class DonationCampaignPageFactory(factory.django.DjangoModelFactory):
    title = factory.Sequence(lambda n: "donation_campaign_page_%d" % n)
    slug = factory.Sequence(lambda n: "donation_campaign_page_%d" % n)
    thank_you_email_subject = ""
    thank_you_email_content = ""

    class Meta:
        model = TestDonationCampaignPage

    @classmethod
    def _create(cls, model_class, *args, **kwargs):
        root_page = Page.objects.all().order_by("depth").first()
        donation_campaign_page = model_class(*args, **kwargs)
        root_page.add_child(instance=donation_campaign_page)
        donation_campaign_page.save()
        return donation_campaign_page


class DonationFundraisingPayInPageFactory(factory.django.DjangoModelFactory):
    title = factory.Sequence(lambda n: "donation_campaign_page_%d" % n)
    slug = factory.Sequence(lambda n: "donation_campaign_page_%d" % n)
    thank_you_email_subject = ""
    thank_you_email_content = ""

    class Meta:
        model = TestDonationFundraisingPayInPage

    @classmethod
    def _create(cls, model_class, *args, **kwargs):
        root_page = Page.objects.all().order_by("depth").first()
        donation_fundraising_pay_in_page = model_class(*args, **kwargs)
        root_page.add_child(instance=donation_fundraising_pay_in_page)
        donation_fundraising_pay_in_page.save()
        return donation_fundraising_pay_in_page


class DonationFactory(factory.django.DjangoModelFactory):
    transaction_id = factory.fuzzy.FuzzyText(
        length=6, chars=string.digits + string.ascii_letters
    )
    transaction_status = ""
    amount = Decimal(10.00)

    first_name = factory.Faker("first_name", locale="en_GB")
    surname = factory.Faker("last_name", locale="en_GB")
    email = factory.Faker("safe_email", locale="en_GB")
    phone_number = factory.Faker("phone_number", locale="en_GB")

    address_line_1 = factory.Faker("secondary_address", locale="en_GB")
    address_line_2 = factory.Faker("street_address", locale="en_GB")
    town = factory.Faker("city", locale="en_GB")
    postal_code = factory.Faker("postcode", locale="en_GB")
    country = "GB"

    on_behalf_of_organisation = False
    gift_aid_declaration = False
    in_memory = False

    class Meta:
        model = SingleDonation



import logging

from django.core.mail import EmailMultiAlternatives
from django.db import models
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _

from wagtail import blocks
from wagtail.admin.panels import (
    FieldPanel,
    InlinePanel,
    MultiFieldPanel,
    ObjectList,
    TabbedInterface,
)
from wagtail.contrib.routable_page.models import RoutablePageMixin, route
from wagtail.fields import StreamField
from wagtail.images import get_image_model_string
from wagtail.models import Orderable, Page

import premailer

from wagtaildonate import constants
from wagtaildonate.admin_forms import DonationAdminForm
from wagtaildonate.settings import donate_settings
from wagtaildonate.utils.blocks import ThankYouBlock
from wagtaildonate.utils.rich_text import rich_text_to_plain_text

image_model = get_image_model_string()

logger = logging.getLogger(__name__)


class EmailParagraphBlock(blocks.RichTextBlock):
    class Meta:
        features = ["bold", "italic", "link", "ol", "ul"]

    def render_as_text(self, value, context=None):
        """
        Renders the paragraph, then converts the resulting HTML into text.
        """
        return rich_text_to_plain_text(self.render_basic(value, context=context))


class EmailStreamBlock(blocks.StreamBlock):
    paragraph = EmailParagraphBlock()

    def render_as_text(self, value, context=None):
        def render_child(child):
            if hasattr(child.block, "render_as_text"):
                return child.block.render_as_text(child.value, context=context)
            else:
                return child.block.render_basic(child.value, context=context)

        return "\n\n".join(map(render_child, value))


class SuggestedAmount(Orderable):
    amount = models.IntegerField()
    panels = [FieldPanel("amount")]

    def __str__(self):
        return str(self.amount)

    class Meta:
        abstract = True


class AbstractDonationLandingPage(Page):
    class Meta:
        abstract = True


class ThankYouRoutablePageMixin(RoutablePageMixin):
    """
    When a donation is completed, the user is presented with a 'Thank You' page.
    The values show on the thank you page will be set on the donation page.
    Using RoutablePageMixin, provide the donation pages with the functionality
    to render these valuse at /thank-you. Also ensure that the thank you page
    can be previewed.
    """

    preview_modes = [("/", "Page"), ("/thank-you/", "Thank you page")]

    thank_you_template = "wagtaildonate/thank_you.html"

    def get_thank_you_data(self):
        """
        Get thank you page content with fallback from the default donation page.
        """
        from wagtaildonate.models import get_default_donation_page

        thank_you_data = {}
        default_donation_page = get_default_donation_page()
        field_names = ["thank_you_strapline", "thank_you_page_content"]
        for field_name in field_names:
            # Try the current object first and revert to the default page.
            field_value = getattr(self, field_name, None)
            if not field_value:
                field_value = getattr(default_donation_page, field_name, None)
            if field_value:
                thank_you_data[field_name] = field_value
        return thank_you_data

    @route(r"^thank-you/$")
    def thank_you_page(self, request, *args, **kwargs):
        if (
            "donation_id" not in request.session
            or "donation_name" not in request.session
            or "donation_amount" not in request.session
        ):
            return HttpResponseRedirect(self.get_url(request))

        thank_you_data = self.get_thank_you_data()
        donation_amount = request.session.pop("donation_amount")
        donation_name = request.session.pop("donation_name")
        in_memory_of = request.session.pop("in_memory_of", False)
        gift_aid_declaration = request.session.pop("gift_aid_declaration", False)
        del request.session["donation_id"]

        return render(
            request,
            self.thank_you_template,
            {
                "page": self,
                "thank_you_data": thank_you_data,
                "donation_amount": donation_amount,
                "donation_name": donation_name,
                "in_memory_of": in_memory_of,
                "gift_aid_declaration": gift_aid_declaration,
            },
        )

    def get_thank_you_page_url(self, request=None):
        thank_you_path = self.reverse_subpage("thank_you_page")
        base_url = self.get_url(request)
        if not base_url.endswith("/"):
            base_url += "/"
        return base_url + thank_you_path

    def serve_preview(self, request, mode_name):
        view, args, kwargs = self.resolve_subpage(mode_name)
        request.is_preview = True

        # Add session data for a dummy preview
        request.session["donation_id"] = "0"
        request.session["donation_amount"] = "10.00"
        request.session["donation_name"] = "John Smith"
        request.session["in_memory_of"] = "Joe Bloggs"
        request.session["gift_aid_declaration"] = True

        return view(request, *args, **kwargs)

    def prepare_request_for_thank_you_page(self, request, donation):
        """
        Save data in session so they can be read by the thank you page.
        """
        request.session["donation_id"] = donation.pk
        request.session["donation_name"] = f"{donation.first_name} {donation.surname}"
        # Format amount to string before saving to session as the default JSON
        # serializer does not like Decimal objects.
        request.session["donation_amount"] = "{:.2f}".format(donation.amount)
        request.session["in_memory_of"] = donation.in_memory_of
        request.session["gift_aid_declaration"] = getattr(
            donation, "gift_aid_declaration", None
        )


class ThankYouEmailMixin(models.Model):
    thank_you_email_subject = models.CharField(max_length=255, blank=True)
    thank_you_email_content = StreamField(
        EmailStreamBlock(required=False), blank=True, use_json_field=True
    )

    class Meta:
        abstract = True

    thank_you_email_email_panels = [
        FieldPanel("thank_you_email_subject"),
        FieldPanel("thank_you_email_content"),
    ]

    def get_thank_you_email_data(self):
        """
        Get thank you page content with fallback from the default donation page.
        """
        from wagtaildonate.models import get_default_donation_page

        thank_you_data = {}
        default_donation_page = get_default_donation_page()
        field_names = ["thank_you_email_content", "thank_you_email_subject"]
        for field_name in field_names:
            # Try the current object first and revert to the default page.
            field_value = getattr(self, field_name, None)
            if not field_value:
                field_value = getattr(default_donation_page, field_name, None)
            if field_value:
                thank_you_data[field_name] = field_value
        return thank_you_data

    def render_email_text(self, donation, thank_you_email_data):
        context = {"donation": donation}
        thank_you_email_content = thank_you_email_data.get("thank_you_email_content")
        if thank_you_email_content:
            block = thank_you_email_content.stream_block
            context["content"] = block.render_as_text(
                thank_you_email_content, context=context
            )
        return render_to_string("wagtaildonate/email/thank_you.txt", context)

    def render_email_html(self, donation, thank_you_email_data):
        context = {"donation": donation}
        thank_you_email_content = thank_you_email_data.get("thank_you_email_content")
        if thank_you_email_content:
            block = thank_you_email_content.stream_block
            context["content"] = block.render(thank_you_email_content, context=context)
        return premailer.transform(
            render_to_string("wagtaildonate/email/thank_you.html", context)
        )

    def prepare_thank_you_email(self, donation):
        thank_you_email_data = self.get_thank_you_email_data()
        message = EmailMultiAlternatives(
            thank_you_email_data.get("thank_you_email_subject", _("Thank you")),
            self.render_email_text(donation, thank_you_email_data),
            to=[donation.email],
        )
        message.attach_alternative(
            self.render_email_html(donation, thank_you_email_data), "text/html"
        )
        return message

    def send_thank_you_email(self, donation):
        if donation.email:
            message = self.prepare_thank_you_email(donation)
            message.send()
            logger.info(
                "Sent thank you email to %s for %s (ID: %d)",
                message.to,
                type(donation).__name__,
                donation.pk,
            )


def get_min_max_amounts_for_context(frequency):
    min_amount = donate_settings.MINIMUM_AMOUNT_PER_FREQUENCY.get(frequency)
    max_amount = donate_settings.MAXIMUM_AMOUNT_PER_FREQUENCY.get(frequency)
    return {
        f"minimum_{frequency}_donation_amount": min_amount,
        f"maximum_{frequency}_donation_amount": max_amount,
    }


class DonationLandingPageBase(
    ThankYouEmailMixin, ThankYouRoutablePageMixin, AbstractDonationLandingPage
):
    """The abstract base class for other donation page classes
    to extend."""

    base_form_class = DonationAdminForm

    introduction = models.TextField(blank=True)
    hero_image = models.ForeignKey(
        image_model, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )
    hero_summary_text = models.TextField(blank=True)
    allow_single_donations = models.BooleanField(default=False)
    allow_regular_donations = models.BooleanField(default=False)
    show_other_amount_option = models.BooleanField(
        default=False,
        help_text=_("Offer the user the option to specify a custom amount."),
    )
    default_donation_frequency = models.CharField(
        max_length=20,
        choices=constants.FREQUENCY_CHOICES,
        default=constants.SINGLE,
        help_text=_(
            "Default donation frequency to prompt the user with. "
            "Only applies if both single and regular donations are allowed."
        ),
    )
    thank_you_strapline = models.TextField(blank=True)
    thank_you_page_content = StreamField(
        ThankYouBlock(required=False), blank=True, use_json_field=True
    )

    # Checkout view specific
    checkout_hero_image = models.ForeignKey(
        image_model, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )
    checkout_hero_summary_text = models.TextField(blank=True)

    content_panels = Page.content_panels + [
        FieldPanel("introduction"),
        MultiFieldPanel(
            [FieldPanel("hero_image"), FieldPanel("hero_summary_text")],
            heading="Hero",
        ),
        MultiFieldPanel(
            [
                FieldPanel("allow_single_donations"),
                FieldPanel("allow_regular_donations"),
                FieldPanel("default_donation_frequency"),
                FieldPanel("show_other_amount_option"),
                InlinePanel(
                    "suggested_amounts_single",
                    label="Suggested single amounts",
                    max_num=3,
                ),
                InlinePanel(
                    "suggested_amounts_regular",
                    label="Suggested regular amounts",
                    max_num=3,
                ),
            ],
            heading="Donation configuration",
        ),
        MultiFieldPanel(
            [
                FieldPanel("checkout_hero_image"),
                FieldPanel("checkout_hero_summary_text"),
            ],
            heading="Checkout Page",
        ),
    ]

    thank_you_panels = [
        FieldPanel("thank_you_strapline"),
        FieldPanel("thank_you_page_content"),
    ]

    edit_handler = TabbedInterface(
        [
            ObjectList(content_panels, heading="Content"),
            ObjectList(thank_you_panels, heading="Thank you page settings"),
            ObjectList(
                ThankYouEmailMixin.thank_you_email_email_panels,
                heading="Email settings",
            ),
        ]
    )

    def get_donation_values(self):
        return {
            "suggested_amounts_single": self.suggested_amounts_single.all(),
            "suggested_amounts_regular": self.suggested_amounts_regular.all(),
        }

    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        context.update(self.get_donation_values())
        context.update(get_min_max_amounts_for_context("single"))
        context.update(get_min_max_amounts_for_context("monthly"))
        return context

    class Meta:
        abstract = True


class AbstractDonationPage(DonationLandingPageBase):
    """The DonationPage acts as the default Donate page.
    Only one of these can exists. It's to ensure that we can
    fall back to values not specified on individual
    campgain/fundraising pages and also so there is always a
    donation method available"""

    max_count = 1

    class Meta:
        abstract = True


class AbstractDonationCampaignPage(DonationLandingPageBase):
    """Similar to DonationLandingPage but more than one can exist"""

    template_name = "wagtaildonate/donation_campaign_page.html"

    class Meta:
        abstract = True


class AbstractPayInLandingPage(Page):
    class Meta:
        abstract = True


class AbstractDonationFundraisingPayInPage(
    ThankYouEmailMixin, ThankYouRoutablePageMixin, AbstractPayInLandingPage
):
    """Simple one-off form for fundraising pay-in
    This does not require any amount fields so only extends Page"""

    introduction = models.TextField(blank=True)
    hero_image = models.ForeignKey(
        image_model, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )
    hero_summary_text = models.TextField(blank=True)
    thank_you_strapline = models.TextField(blank=True)
    thank_you_page_content = StreamField(
        ThankYouBlock(required=False), blank=True, use_json_field=True
    )

    content_panels = Page.content_panels + [
        FieldPanel("introduction"),
        MultiFieldPanel(
            [FieldPanel("hero_image"), FieldPanel("hero_summary_text")],
            heading="Hero",
        ),
    ]

    thank_you_panels = [
        FieldPanel("thank_you_strapline"),
        FieldPanel("thank_you_page_content"),
    ]

    edit_handler = TabbedInterface(
        [
            ObjectList(content_panels, heading="Content"),
            ObjectList(thank_you_panels, heading="Thank you page settings"),
            ObjectList(
                ThankYouEmailMixin.thank_you_email_email_panels,
                heading="Email settings",
            ),
        ]
    )

    thank_you_template = "wagtaildonate/thank_you_fundraising.html"

    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        context.update(get_min_max_amounts_for_context("payin"))
        return context

    class Meta:
        abstract = True



from django.test import TestCase, override_settings


@override_settings(
    WAGTAIL_DONATE={
        "BRAINTREE_SANDBOX": True,
        "BRAINTREE_PUBLIC_KEY": "mock-public-key",
        "BRAINTREE_PRIVATE_KEY": "mock-private-key",
        "BRAINTREE_MERCHANT_ID": "mock-merchan-it",
    }
)
class TestBraintreeUtils(TestCase):
    def test_get_config(self):
        pass

    def test_get_gateway(self):
        pass

    def test_get_client_token(self):
        pass



# Generated by Django 2.2.12 on 2020-06-01 10:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0029_rename_donation_model_to_single_donation"),
    ]

    operations = [
        migrations.AddField(
            model_name="payin",
            name="address_line_3",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Address line 3"
            ),
        ),
        migrations.AddField(
            model_name="recurringdonation",
            name="address_line_3",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Address line 3"
            ),
        ),
        migrations.AddField(
            model_name="singledonation",
            name="address_line_3",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Address line 3"
            ),
        ),
    ]



import logging

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from phonenumber_field.modelfields import PhoneNumberField

from wagtaildonate.countries import country_choices
from wagtaildonate.models.query import DonationQuerySet
from wagtaildonate.payment_methods.base import PaymentMethod
from wagtaildonate.payment_methods.utils import get_payment_method
from wagtaildonate.utils.currency import format_gbp

logger = logging.getLogger(__name__)


class OptIns(models.Model):
    sms_consent = models.BooleanField(
        null=True, default=None, blank=True, verbose_name=_("SMS consent")
    )
    phone_consent = models.BooleanField(null=True, default=None, blank=True)
    post_consent = models.BooleanField(null=True, default=None, blank=True)
    email_consent = models.BooleanField(null=True, blank=True, default=None)

    class Meta:
        abstract = True


class GiftAid(models.Model):
    gift_aid_declaration = models.BooleanField(_("Gift Aid"))

    class Meta:
        abstract = True


class TransactionMixin(models.Model):
    STATUS_SETTLING = "settling"
    STATUS_SETTLED = "settled"
    STATUS_FAILED = "failed"  # Refer to provider_transaction_status for reason
    STATUS_UNKNOWN = "unknown"  # Refer to provider_transaction_status
    STATUS_CHOICES = [
        (STATUS_SETTLING, _("Settling")),
        (STATUS_SETTLED, _("Settled")),
        (STATUS_FAILED, _("Failed")),
        (STATUS_UNKNOWN, _("Unknown")),
    ]

    transaction_id = models.CharField(_("Transaction ID"), max_length=50)
    transaction_status = models.CharField(max_length=100, choices=STATUS_CHOICES)
    # If the payment provider returned a more specific or unknown status. Refer to this field for details
    provider_transaction_status = models.CharField(max_length=100, blank=True)

    # 3D Secure related fields
    # liability_shifted indicates that 3D Secure verification has taken place successfully
    # and fraud liability has been shifted from the merchant to the card issuer.
    # liability_shift_possible indicates that the card is enrolled in 3D Secure and liabilty
    # can be shifted from merchant to issuer.
    # three_d_secure_status holds the 3D Secure verification result from the provider.
    liability_shifted = models.BooleanField(blank=True, default=False)
    liability_shift_possible = models.BooleanField(blank=True, default=False)
    three_d_secure_status = models.CharField(max_length=100, blank=True, default="")

    is_recurring = False
    objects = DonationQuerySet.as_manager()

    class Meta:
        abstract = True

    def is_settling(self) -> bool:
        return self.transaction_status == self.STATUS_SETTLING

    def update_settlement_status(
        self, commit: bool = True, output_non_commit_log: bool = False
    ):
        payment_method = self.get_payment_method()
        new_status = payment_method.get_transaction_status_for_transaction(
            self.transaction_id
        )
        if self.provider_transaction_status == new_status.provider_transaction_status:
            # No need to update, already the same status.
            return
        old_status = self.transaction_status
        old_provider_status = self.provider_transaction_status
        self.provider_transaction_status = new_status.provider_transaction_status
        self.transaction_status = new_status.transaction_status
        if commit:
            logger.info(
                "Update transaction status for %s (ID: %d), %s (%s) -> %s (%s)",
                type(self).__name__,
                self.pk,
                old_status,
                old_provider_status,
                new_status.transaction_status,
                new_status.provider_transaction_status,
            )
            self.save(
                update_fields=["provider_transaction_status", "transaction_status"]
            )
        elif output_non_commit_log:
            logger.info(
                "Non-commit update transaction status for %s (ID: %d), %s (%s) -> %s (%s), %s (ID: %s)",
                type(self).__name__,
                self.pk,
                old_status,
                old_provider_status,
                new_status.transaction_status,
                new_status.provider_transaction_status,
                self.payment_method,
                self.transaction_id,
            )


class BaseDonation(OptIns, models.Model):
    donation_page = models.ForeignKey(
        "wagtailcore.Page", on_delete=models.SET_NULL, null=True, related_name="+"
    )
    payment_method = models.CharField(max_length=100)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    first_name = models.CharField(_("First name"), max_length=100)
    surname = models.CharField(_("Surname"), max_length=100)
    email = models.EmailField(_("Email address"), max_length=255, blank=True)
    phone_number = PhoneNumberField(_("Phone number"), blank=True)
    address_line_1 = models.CharField(_("Address line 1"), max_length=255)
    address_line_2 = models.CharField(_("Address line 2"), max_length=255, blank=True)
    address_line_3 = models.CharField(_("Address line 3"), max_length=255, blank=True)
    town = models.CharField(_("Town"), max_length=255)
    postal_code = models.CharField(_("Postal code"), max_length=50)
    country = models.CharField(_("Country"), max_length=255, choices=country_choices())
    on_behalf_of_organisation = models.BooleanField(_("On behalf of organisation"))
    in_memory = models.BooleanField(_("In memory"))
    in_memory_of = models.CharField(_("In memory of"), max_length=60, blank=True)
    created_at = models.DateTimeField(_("Created at"), default=timezone.now)

    def save(self, *args, **kwargs):
        # If there is no pk, thus meaning that this donation is being created
        # log the donation as succesful.
        if self.pk is None:
            super().save(*args, **kwargs)
            logger.info("%s created (ID: %d)", type(self).__name__, self.pk)
            return
        super().save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        logger.info("%s deleted (ID: %d)", type(self).__name__, self.pk)
        return super().delete(*args, **kwargs)

    def formatted_amount(self) -> str:
        return format_gbp(self.amount)

    def get_payment_method(self) -> PaymentMethod:
        return get_payment_method(self.payment_method)

    class Meta:
        abstract = True


class AbstractSingleDonation(GiftAid, TransactionMixin, BaseDonation):
    class Meta:
        abstract = True
        verbose_name = _("Donation")
        verbose_name_plural = _("Donations")
        permissions = (("can_export_donations", _("Can export donate transactions")),)


class AbstractRecurringDonation(GiftAid, BaseDonation):
    FREQUENCY_MONTHLY = "monthly"
    FREQUENCY_CHOICES = [
        (FREQUENCY_MONTHLY, _("Monthly")),
    ]

    subscription_id = models.CharField(_("Subscription ID"), max_length=50)
    frequency = models.CharField(max_length=100, choices=FREQUENCY_CHOICES)

    is_recurring = True

    class Meta:
        abstract = True
        verbose_name = _("Recurring Donation")
        verbose_name_plural = _("Recurring Donations")
        permissions = (
            (
                "can_export_recurring_donations",
                _("Can export recurring donate transactions"),
            ),
        )


class AbstractPayIn(TransactionMixin, BaseDonation):
    """
    Pay ins are very similar to one-off donations with an exception of not
    supporting Gift Aid and of allowing pay in event details to be stored
    """

    event_name = models.CharField(max_length=255, blank=True)
    event_code = models.CharField(max_length=50, blank=True)
    fundraiser_reference = models.CharField(max_length=50, blank=True)

    class Meta:
        abstract = True
        permissions = (("can_export_pay_ins", _("Can export pay ins")),)


class SingleDonation(AbstractSingleDonation):
    pass


class RecurringDonation(AbstractRecurringDonation):
    pass


class PayIn(AbstractPayIn):
    pass



# Generated by Django 2.2.11 on 2020-03-13 11:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0008_donation_giftaid_organisation"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="in_memory_of",
            field=models.CharField(
                blank=True, default="", max_length=60, verbose_name="In memory of"
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="in_memory",
            field=models.BooleanField(default=False, verbose_name="In memory"),
            preserve_default=False,
        ),
    ]



from wagtaildonate.address_lookups.base import AddressLookup


class GenericAddressLookup(AddressLookup):
    code = "generic"

    def get_frontend_options(self):
        return {
            "auto_complete_enabled": self.options.get("AUTO_COMPLETE_ENABLED", True)
        }



from decimal import Decimal
from typing import Dict, Optional, Type

from django.db import models

from wagtail.models import Page

from rest_framework import generics, permissions, serializers, status
from rest_framework.response import Response

from wagtaildonate.api.metadata import HiddenMetadata
from wagtaildonate.api.serializers.payment_method import (
    PaymentMethodAndFrequencySerializer,
)
from wagtaildonate.models import get_default_donation_page
from wagtaildonate.payment_methods.base import PaymentMethod
from wagtaildonate.settings import donate_settings


class DonationsCreateAPIView(generics.CreateAPIView):
    """
    Make donations.
    """

    donation_id: Optional[int]

    # This view is available to all the users.
    permission_classes = [permissions.AllowAny]

    # Make this view CSRF exempt.
    authentication_classes = []

    # Hide data on the OPTIONS request.
    metadata_class = HiddenMetadata

    # Initial serializer that validates payment_method and frequency.
    # It will return appropriate payment method and serializer class for the
    # specific payment method.
    serializer_class = PaymentMethodAndFrequencySerializer

    def __init__(self, *args, **kwargs):
        self.donation_id = None
        super().__init__(*args, **kwargs)

    def create(self, request, *args, **kwargs) -> Response:
        """
        Only return ID when the donation is created.
        https://github.com/encode/django-rest-framework/blob/acbd9d8222e763c7f9c7dc2de23c430c702e06d4/rest_framework/mixins.py#L16-L21
        """
        super().create(request, *args, **kwargs)
        return Response({"id": self.donation_id}, status=status.HTTP_201_CREATED)

    def perform_create(self, serializer: PaymentMethodAndFrequencySerializer) -> None:
        """
        Create donation object after payment method and frequency is correct.
        https://github.com/encode/django-rest-framework/blob/acbd9d8222e763c7f9c7dc2de23c430c702e06d4/rest_framework/mixins.py#L23-L24
        """
        # Get payment method from the serializer.
        self.payment_method = serializer.validated_data["payment_method"]

        # Try to create the donation entry in the Wagtail database.
        # If exception wasn't raised, assume it's created.
        donation = self.create_donation(serializer)

        # Set self.donation_id for use in self.create()
        self.donation_id = donation.pk

    def get_payment_method_serializer_class(
        self, *, payment_method: PaymentMethod, frequency: str, **kwargs
    ) -> Type[serializers.Serializer]:
        """
        Get serializer class for the given payment method and frequency.
        """
        return payment_method.get_serializer_class_for_frequency(frequency)

    def get_payment_method_serializer_context(
        self, *, frequency: str, **kwargs
    ) -> Dict[str, any]:
        """
        Extra context provided to the serializer class context.
        """
        return {
            **self.get_serializer_context(),
            "minimum_amount": self.get_minimum_amount_for_frequency(frequency),
            "maximum_amount": self.get_maximum_amount_for_frequency(frequency),
            **kwargs,
        }

    def get_payment_method_serializer(
        self, *args, payment_method: PaymentMethod, frequency: str, **kwargs
    ) -> serializers.Serializer:
        """
        Instantiate payment method serializer class.
        """
        serializer_class = self.get_payment_method_serializer_class(
            payment_method=payment_method, frequency=frequency
        )
        kwargs.setdefault(
            "context",
            self.get_payment_method_serializer_context(
                frequency=frequency, payment_method=payment_method, **kwargs
            ),
        )
        kwargs.pop("frequency", None)
        return serializer_class(*args, **kwargs)

    def create_donation(
        self, initial_serializer: PaymentMethodAndFrequencySerializer
    ) -> models.Model:
        """
        Call payment method serializer to validate and sanitise the input
        of the donation specific data; and create the donation object.
        """
        payment_serializer = self.get_payment_method_serializer(
            data=self.request.data, **initial_serializer.validated_data
        )
        payment_serializer.is_valid(raise_exception=True)
        donation = self.perform_create_donation(payment_serializer)
        self.perform_after_create_donation(donation)
        return donation

    def get_minimum_amount_for_frequency(self, frequency: str) -> Decimal:
        return Decimal(donate_settings.MINIMUM_AMOUNT_PER_FREQUENCY.get(frequency))

    def get_maximum_amount_for_frequency(self, frequency: str) -> Decimal:
        return Decimal(donate_settings.MAXIMUM_AMOUNT_PER_FREQUENCY.get(frequency))

    def perform_create_donation(
        self, payment_serializer: serializers.Serializer
    ) -> models.Model:
        """
        Save the donation object to the database.
        """
        return payment_serializer.save()

    def get_donation_page(self, donation: models.Model) -> Page:
        """
        Get donation page that is associated with the donation.
        """
        if donation.donation_page:
            return donation.donation_page.specific
        return get_default_donation_page().specific

    def perform_after_create_donation(self, donation: models.Model) -> None:
        """
        Additional tasks that happen after creating a donation.
        """
        self.prepare_request_for_thank_you_page(donation)
        self.send_thank_you_email(donation)

    def send_thank_you_email(self, donation: models.Model) -> None:
        donation_page = self.get_donation_page(donation)
        if hasattr(donation_page, "send_thank_you_email"):
            donation_page.send_thank_you_email(donation)

    def prepare_request_for_thank_you_page(self, donation: models.Model) -> None:
        donation_page = self.get_donation_page(donation)
        if hasattr(donation_page, "prepare_request_for_thank_you_page"):
            donation_page.prepare_request_for_thank_you_page(self.request, donation)



# Generated by Django 5.0.6 on 2024-05-22 04:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0034_alter_thankyouctasnippet_link"),
    ]

    operations = [
        migrations.AlterField(
            model_name="payin",
            name="country",
            field=models.CharField(
                choices=[
                    ("GB", "United Kingdom"),
                    ("IE", "Ireland"),
                    ("AF", "Afghanistan"),
                    ("AL", "Albania"),
                    ("DZ", "Algeria"),
                    ("AS", "American Samoa"),
                    ("AD", "Andorra"),
                    ("AO", "Angola"),
                    ("AI", "Anguilla"),
                    ("AQ", "Antarctica"),
                    ("AG", "Antigua and Barbuda"),
                    ("AR", "Argentina"),
                    ("AM", "Armenia"),
                    ("AW", "Aruba"),
                    ("AU", "Australia"),
                    ("AT", "Austria"),
                    ("AZ", "Azerbaijan"),
                    ("BS", "Bahamas"),
                    ("BH", "Bahrain"),
                    ("BD", "Bangladesh"),
                    ("BB", "Barbados"),
                    ("BY", "Belarus"),
                    ("BE", "Belgium"),
                    ("BZ", "Belize"),
                    ("BJ", "Benin"),
                    ("BM", "Bermuda"),
                    ("BT", "Bhutan"),
                    ("BO", "Bolivia, Plurinational State of"),
                    ("BQ", "Bonaire, Sint Eustatius and Saba"),
                    ("BA", "Bosnia and Herzegovina"),
                    ("BW", "Botswana"),
                    ("BV", "Bouvet Island"),
                    ("BR", "Brazil"),
                    ("IO", "British Indian Ocean Territory"),
                    ("BN", "Brunei Darussalam"),
                    ("BG", "Bulgaria"),
                    ("BF", "Burkina Faso"),
                    ("BI", "Burundi"),
                    ("CV", "Cabo Verde"),
                    ("KH", "Cambodia"),
                    ("CM", "Cameroon"),
                    ("CA", "Canada"),
                    ("KY", "Cayman Islands"),
                    ("CF", "Central African Republic"),
                    ("TD", "Chad"),
                    ("CL", "Chile"),
                    ("CN", "China"),
                    ("CX", "Christmas Island"),
                    ("CC", "Cocos (Keeling) Islands"),
                    ("CO", "Colombia"),
                    ("KM", "Comoros"),
                    ("CG", "Congo"),
                    ("CD", "Congo, The Democratic Republic of the"),
                    ("CK", "Cook Islands"),
                    ("CR", "Costa Rica"),
                    ("HR", "Croatia"),
                    ("CU", "Cuba"),
                    ("CW", "Curaçao"),
                    ("CY", "Cyprus"),
                    ("CZ", "Czechia"),
                    ("CI", "Côte d'Ivoire"),
                    ("DK", "Denmark"),
                    ("DJ", "Djibouti"),
                    ("DM", "Dominica"),
                    ("DO", "Dominican Republic"),
                    ("EC", "Ecuador"),
                    ("EG", "Egypt"),
                    ("SV", "El Salvador"),
                    ("GQ", "Equatorial Guinea"),
                    ("ER", "Eritrea"),
                    ("EE", "Estonia"),
                    ("SZ", "Eswatini"),
                    ("ET", "Ethiopia"),
                    ("FK", "Falkland Islands (Malvinas)"),
                    ("FO", "Faroe Islands"),
                    ("FJ", "Fiji"),
                    ("FI", "Finland"),
                    ("FR", "France"),
                    ("GF", "French Guiana"),
                    ("PF", "French Polynesia"),
                    ("TF", "French Southern Territories"),
                    ("GA", "Gabon"),
                    ("GM", "Gambia"),
                    ("GE", "Georgia"),
                    ("DE", "Germany"),
                    ("GH", "Ghana"),
                    ("GI", "Gibraltar"),
                    ("GR", "Greece"),
                    ("GL", "Greenland"),
                    ("GD", "Grenada"),
                    ("GP", "Guadeloupe"),
                    ("GU", "Guam"),
                    ("GT", "Guatemala"),
                    ("GG", "Guernsey"),
                    ("GN", "Guinea"),
                    ("GW", "Guinea-Bissau"),
                    ("GY", "Guyana"),
                    ("HT", "Haiti"),
                    ("HM", "Heard Island and McDonald Islands"),
                    ("VA", "Holy See (Vatican City State)"),
                    ("HN", "Honduras"),
                    ("HK", "Hong Kong"),
                    ("HU", "Hungary"),
                    ("IS", "Iceland"),
                    ("IN", "India"),
                    ("ID", "Indonesia"),
                    ("IR", "Iran, Islamic Republic of"),
                    ("IQ", "Iraq"),
                    ("IM", "Isle of Man"),
                    ("IL", "Israel"),
                    ("IT", "Italy"),
                    ("JM", "Jamaica"),
                    ("JP", "Japan"),
                    ("JE", "Jersey"),
                    ("JO", "Jordan"),
                    ("KZ", "Kazakhstan"),
                    ("KE", "Kenya"),
                    ("KI", "Kiribati"),
                    ("KP", "Korea, Democratic People's Republic of"),
                    ("KR", "Korea, Republic of"),
                    ("KW", "Kuwait"),
                    ("KG", "Kyrgyzstan"),
                    ("LA", "Lao People's Democratic Republic"),
                    ("LV", "Latvia"),
                    ("LB", "Lebanon"),
                    ("LS", "Lesotho"),
                    ("LR", "Liberia"),
                    ("LY", "Libya"),
                    ("LI", "Liechtenstein"),
                    ("LT", "Lithuania"),
                    ("LU", "Luxembourg"),
                    ("MO", "Macao"),
                    ("MG", "Madagascar"),
                    ("MW", "Malawi"),
                    ("MY", "Malaysia"),
                    ("MV", "Maldives"),
                    ("ML", "Mali"),
                    ("MT", "Malta"),
                    ("MH", "Marshall Islands"),
                    ("MQ", "Martinique"),
                    ("MR", "Mauritania"),
                    ("MU", "Mauritius"),
                    ("YT", "Mayotte"),
                    ("MX", "Mexico"),
                    ("FM", "Micronesia, Federated States of"),
                    ("MD", "Moldova, Republic of"),
                    ("MC", "Monaco"),
                    ("MN", "Mongolia"),
                    ("ME", "Montenegro"),
                    ("MS", "Montserrat"),
                    ("MA", "Morocco"),
                    ("MZ", "Mozambique"),
                    ("MM", "Myanmar"),
                    ("NA", "Namibia"),
                    ("NR", "Nauru"),
                    ("NP", "Nepal"),
                    ("NL", "Netherlands"),
                    ("NC", "New Caledonia"),
                    ("NZ", "New Zealand"),
                    ("NI", "Nicaragua"),
                    ("NE", "Niger"),
                    ("NG", "Nigeria"),
                    ("NU", "Niue"),
                    ("NF", "Norfolk Island"),
                    ("MK", "North Macedonia"),
                    ("MP", "Northern Mariana Islands"),
                    ("NO", "Norway"),
                    ("OM", "Oman"),
                    ("PK", "Pakistan"),
                    ("PW", "Palau"),
                    ("PS", "Palestine, State of"),
                    ("PA", "Panama"),
                    ("PG", "Papua New Guinea"),
                    ("PY", "Paraguay"),
                    ("PE", "Peru"),
                    ("PH", "Philippines"),
                    ("PN", "Pitcairn"),
                    ("PL", "Poland"),
                    ("PT", "Portugal"),
                    ("PR", "Puerto Rico"),
                    ("QA", "Qatar"),
                    ("RO", "Romania"),
                    ("RU", "Russian Federation"),
                    ("RW", "Rwanda"),
                    ("RE", "Réunion"),
                    ("BL", "Saint Barthélemy"),
                    ("SH", "Saint Helena, Ascension and Tristan da Cunha"),
                    ("KN", "Saint Kitts and Nevis"),
                    ("LC", "Saint Lucia"),
                    ("MF", "Saint Martin (French part)"),
                    ("PM", "Saint Pierre and Miquelon"),
                    ("VC", "Saint Vincent and the Grenadines"),
                    ("WS", "Samoa"),
                    ("SM", "San Marino"),
                    ("ST", "Sao Tome and Principe"),
                    ("SA", "Saudi Arabia"),
                    ("SN", "Senegal"),
                    ("RS", "Serbia"),
                    ("SC", "Seychelles"),
                    ("SL", "Sierra Leone"),
                    ("SG", "Singapore"),
                    ("SX", "Sint Maarten (Dutch part)"),
                    ("SK", "Slovakia"),
                    ("SI", "Slovenia"),
                    ("SB", "Solomon Islands"),
                    ("SO", "Somalia"),
                    ("ZA", "South Africa"),
                    ("GS", "South Georgia and the South Sandwich Islands"),
                    ("SS", "South Sudan"),
                    ("ES", "Spain"),
                    ("LK", "Sri Lanka"),
                    ("SD", "Sudan"),
                    ("SR", "Suriname"),
                    ("SJ", "Svalbard and Jan Mayen"),
                    ("SE", "Sweden"),
                    ("CH", "Switzerland"),
                    ("SY", "Syrian Arab Republic"),
                    ("TW", "Taiwan, Province of China"),
                    ("TJ", "Tajikistan"),
                    ("TZ", "Tanzania, United Republic of"),
                    ("TH", "Thailand"),
                    ("TL", "Timor-Leste"),
                    ("TG", "Togo"),
                    ("TK", "Tokelau"),
                    ("TO", "Tonga"),
                    ("TT", "Trinidad and Tobago"),
                    ("TN", "Tunisia"),
                    ("TM", "Turkmenistan"),
                    ("TC", "Turks and Caicos Islands"),
                    ("TV", "Tuvalu"),
                    ("TR", "Türkiye"),
                    ("UG", "Uganda"),
                    ("UA", "Ukraine"),
                    ("AE", "United Arab Emirates"),
                    ("US", "United States"),
                    ("UM", "United States Minor Outlying Islands"),
                    ("UY", "Uruguay"),
                    ("UZ", "Uzbekistan"),
                    ("VU", "Vanuatu"),
                    ("VE", "Venezuela, Bolivarian Republic of"),
                    ("VN", "Viet Nam"),
                    ("VG", "Virgin Islands, British"),
                    ("VI", "Virgin Islands, U.S."),
                    ("WF", "Wallis and Futuna"),
                    ("EH", "Western Sahara"),
                    ("YE", "Yemen"),
                    ("ZM", "Zambia"),
                    ("ZW", "Zimbabwe"),
                    ("AX", "Åland Islands"),
                ],
                max_length=255,
                verbose_name="Country",
            ),
        ),
        migrations.AlterField(
            model_name="recurringdonation",
            name="country",
            field=models.CharField(
                choices=[
                    ("GB", "United Kingdom"),
                    ("IE", "Ireland"),
                    ("AF", "Afghanistan"),
                    ("AL", "Albania"),
                    ("DZ", "Algeria"),
                    ("AS", "American Samoa"),
                    ("AD", "Andorra"),
                    ("AO", "Angola"),
                    ("AI", "Anguilla"),
                    ("AQ", "Antarctica"),
                    ("AG", "Antigua and Barbuda"),
                    ("AR", "Argentina"),
                    ("AM", "Armenia"),
                    ("AW", "Aruba"),
                    ("AU", "Australia"),
                    ("AT", "Austria"),
                    ("AZ", "Azerbaijan"),
                    ("BS", "Bahamas"),
                    ("BH", "Bahrain"),
                    ("BD", "Bangladesh"),
                    ("BB", "Barbados"),
                    ("BY", "Belarus"),
                    ("BE", "Belgium"),
                    ("BZ", "Belize"),
                    ("BJ", "Benin"),
                    ("BM", "Bermuda"),
                    ("BT", "Bhutan"),
                    ("BO", "Bolivia, Plurinational State of"),
                    ("BQ", "Bonaire, Sint Eustatius and Saba"),
                    ("BA", "Bosnia and Herzegovina"),
                    ("BW", "Botswana"),
                    ("BV", "Bouvet Island"),
                    ("BR", "Brazil"),
                    ("IO", "British Indian Ocean Territory"),
                    ("BN", "Brunei Darussalam"),
                    ("BG", "Bulgaria"),
                    ("BF", "Burkina Faso"),
                    ("BI", "Burundi"),
                    ("CV", "Cabo Verde"),
                    ("KH", "Cambodia"),
                    ("CM", "Cameroon"),
                    ("CA", "Canada"),
                    ("KY", "Cayman Islands"),
                    ("CF", "Central African Republic"),
                    ("TD", "Chad"),
                    ("CL", "Chile"),
                    ("CN", "China"),
                    ("CX", "Christmas Island"),
                    ("CC", "Cocos (Keeling) Islands"),
                    ("CO", "Colombia"),
                    ("KM", "Comoros"),
                    ("CG", "Congo"),
                    ("CD", "Congo, The Democratic Republic of the"),
                    ("CK", "Cook Islands"),
                    ("CR", "Costa Rica"),
                    ("HR", "Croatia"),
                    ("CU", "Cuba"),
                    ("CW", "Curaçao"),
                    ("CY", "Cyprus"),
                    ("CZ", "Czechia"),
                    ("CI", "Côte d'Ivoire"),
                    ("DK", "Denmark"),
                    ("DJ", "Djibouti"),
                    ("DM", "Dominica"),
                    ("DO", "Dominican Republic"),
                    ("EC", "Ecuador"),
                    ("EG", "Egypt"),
                    ("SV", "El Salvador"),
                    ("GQ", "Equatorial Guinea"),
                    ("ER", "Eritrea"),
                    ("EE", "Estonia"),
                    ("SZ", "Eswatini"),
                    ("ET", "Ethiopia"),
                    ("FK", "Falkland Islands (Malvinas)"),
                    ("FO", "Faroe Islands"),
                    ("FJ", "Fiji"),
                    ("FI", "Finland"),
                    ("FR", "France"),
                    ("GF", "French Guiana"),
                    ("PF", "French Polynesia"),
                    ("TF", "French Southern Territories"),
                    ("GA", "Gabon"),
                    ("GM", "Gambia"),
                    ("GE", "Georgia"),
                    ("DE", "Germany"),
                    ("GH", "Ghana"),
                    ("GI", "Gibraltar"),
                    ("GR", "Greece"),
                    ("GL", "Greenland"),
                    ("GD", "Grenada"),
                    ("GP", "Guadeloupe"),
                    ("GU", "Guam"),
                    ("GT", "Guatemala"),
                    ("GG", "Guernsey"),
                    ("GN", "Guinea"),
                    ("GW", "Guinea-Bissau"),
                    ("GY", "Guyana"),
                    ("HT", "Haiti"),
                    ("HM", "Heard Island and McDonald Islands"),
                    ("VA", "Holy See (Vatican City State)"),
                    ("HN", "Honduras"),
                    ("HK", "Hong Kong"),
                    ("HU", "Hungary"),
                    ("IS", "Iceland"),
                    ("IN", "India"),
                    ("ID", "Indonesia"),
                    ("IR", "Iran, Islamic Republic of"),
                    ("IQ", "Iraq"),
                    ("IM", "Isle of Man"),
                    ("IL", "Israel"),
                    ("IT", "Italy"),
                    ("JM", "Jamaica"),
                    ("JP", "Japan"),
                    ("JE", "Jersey"),
                    ("JO", "Jordan"),
                    ("KZ", "Kazakhstan"),
                    ("KE", "Kenya"),
                    ("KI", "Kiribati"),
                    ("KP", "Korea, Democratic People's Republic of"),
                    ("KR", "Korea, Republic of"),
                    ("KW", "Kuwait"),
                    ("KG", "Kyrgyzstan"),
                    ("LA", "Lao People's Democratic Republic"),
                    ("LV", "Latvia"),
                    ("LB", "Lebanon"),
                    ("LS", "Lesotho"),
                    ("LR", "Liberia"),
                    ("LY", "Libya"),
                    ("LI", "Liechtenstein"),
                    ("LT", "Lithuania"),
                    ("LU", "Luxembourg"),
                    ("MO", "Macao"),
                    ("MG", "Madagascar"),
                    ("MW", "Malawi"),
                    ("MY", "Malaysia"),
                    ("MV", "Maldives"),
                    ("ML", "Mali"),
                    ("MT", "Malta"),
                    ("MH", "Marshall Islands"),
                    ("MQ", "Martinique"),
                    ("MR", "Mauritania"),
                    ("MU", "Mauritius"),
                    ("YT", "Mayotte"),
                    ("MX", "Mexico"),
                    ("FM", "Micronesia, Federated States of"),
                    ("MD", "Moldova, Republic of"),
                    ("MC", "Monaco"),
                    ("MN", "Mongolia"),
                    ("ME", "Montenegro"),
                    ("MS", "Montserrat"),
                    ("MA", "Morocco"),
                    ("MZ", "Mozambique"),
                    ("MM", "Myanmar"),
                    ("NA", "Namibia"),
                    ("NR", "Nauru"),
                    ("NP", "Nepal"),
                    ("NL", "Netherlands"),
                    ("NC", "New Caledonia"),
                    ("NZ", "New Zealand"),
                    ("NI", "Nicaragua"),
                    ("NE", "Niger"),
                    ("NG", "Nigeria"),
                    ("NU", "Niue"),
                    ("NF", "Norfolk Island"),
                    ("MK", "North Macedonia"),
                    ("MP", "Northern Mariana Islands"),
                    ("NO", "Norway"),
                    ("OM", "Oman"),
                    ("PK", "Pakistan"),
                    ("PW", "Palau"),
                    ("PS", "Palestine, State of"),
                    ("PA", "Panama"),
                    ("PG", "Papua New Guinea"),
                    ("PY", "Paraguay"),
                    ("PE", "Peru"),
                    ("PH", "Philippines"),
                    ("PN", "Pitcairn"),
                    ("PL", "Poland"),
                    ("PT", "Portugal"),
                    ("PR", "Puerto Rico"),
                    ("QA", "Qatar"),
                    ("RO", "Romania"),
                    ("RU", "Russian Federation"),
                    ("RW", "Rwanda"),
                    ("RE", "Réunion"),
                    ("BL", "Saint Barthélemy"),
                    ("SH", "Saint Helena, Ascension and Tristan da Cunha"),
                    ("KN", "Saint Kitts and Nevis"),
                    ("LC", "Saint Lucia"),
                    ("MF", "Saint Martin (French part)"),
                    ("PM", "Saint Pierre and Miquelon"),
                    ("VC", "Saint Vincent and the Grenadines"),
                    ("WS", "Samoa"),
                    ("SM", "San Marino"),
                    ("ST", "Sao Tome and Principe"),
                    ("SA", "Saudi Arabia"),
                    ("SN", "Senegal"),
                    ("RS", "Serbia"),
                    ("SC", "Seychelles"),
                    ("SL", "Sierra Leone"),
                    ("SG", "Singapore"),
                    ("SX", "Sint Maarten (Dutch part)"),
                    ("SK", "Slovakia"),
                    ("SI", "Slovenia"),
                    ("SB", "Solomon Islands"),
                    ("SO", "Somalia"),
                    ("ZA", "South Africa"),
                    ("GS", "South Georgia and the South Sandwich Islands"),
                    ("SS", "South Sudan"),
                    ("ES", "Spain"),
                    ("LK", "Sri Lanka"),
                    ("SD", "Sudan"),
                    ("SR", "Suriname"),
                    ("SJ", "Svalbard and Jan Mayen"),
                    ("SE", "Sweden"),
                    ("CH", "Switzerland"),
                    ("SY", "Syrian Arab Republic"),
                    ("TW", "Taiwan, Province of China"),
                    ("TJ", "Tajikistan"),
                    ("TZ", "Tanzania, United Republic of"),
                    ("TH", "Thailand"),
                    ("TL", "Timor-Leste"),
                    ("TG", "Togo"),
                    ("TK", "Tokelau"),
                    ("TO", "Tonga"),
                    ("TT", "Trinidad and Tobago"),
                    ("TN", "Tunisia"),
                    ("TM", "Turkmenistan"),
                    ("TC", "Turks and Caicos Islands"),
                    ("TV", "Tuvalu"),
                    ("TR", "Türkiye"),
                    ("UG", "Uganda"),
                    ("UA", "Ukraine"),
                    ("AE", "United Arab Emirates"),
                    ("US", "United States"),
                    ("UM", "United States Minor Outlying Islands"),
                    ("UY", "Uruguay"),
                    ("UZ", "Uzbekistan"),
                    ("VU", "Vanuatu"),
                    ("VE", "Venezuela, Bolivarian Republic of"),
                    ("VN", "Viet Nam"),
                    ("VG", "Virgin Islands, British"),
                    ("VI", "Virgin Islands, U.S."),
                    ("WF", "Wallis and Futuna"),
                    ("EH", "Western Sahara"),
                    ("YE", "Yemen"),
                    ("ZM", "Zambia"),
                    ("ZW", "Zimbabwe"),
                    ("AX", "Åland Islands"),
                ],
                max_length=255,
                verbose_name="Country",
            ),
        ),
    ]



from rest_framework import serializers

from wagtaildonate.api.serializers.utils import get_pay_in_event_serializer_class
from wagtaildonate.models.utils import get_pay_in_event_model


class CountrySerializer(serializers.Serializer):
    code = serializers.CharField(source="alpha_2")
    name = serializers.CharField()


class PaymentMethodSerializer(serializers.Serializer):
    code = serializers.CharField()
    options = serializers.JSONField(source="get_frontend_options")
    supported_frequencies = serializers.ReadOnlyField(
        source="get_supported_frequencies"
    )


class AddressLookupSerializer(serializers.Serializer):
    code = serializers.CharField()
    options = serializers.JSONField(source="get_frontend_options")


class PayInEventSerializer(serializers.ModelSerializer):
    class Meta:
        model = get_pay_in_event_model()
        fields = [
            "event_code",
            "event_name",
            "fundraiser_reference_required",
        ]


class ConfigurationSerializer(serializers.Serializer):
    checkout_url = serializers.URLField()
    address_lookup = AddressLookupSerializer()
    payment_methods = PaymentMethodSerializer(many=True)
    countries = CountrySerializer(many=True)
    pay_in_events = get_pay_in_event_serializer_class()(many=True)
    pay_in_page_id = serializers.IntegerField()
    pay_in_success_url = serializers.URLField()



# Generated by Django 2.2.12 on 2020-05-29 20:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0045_assign_unlock_grouppagepermission"),
        ("wagtaildonate", "0028_payin"),
    ]

    operations = [
        migrations.RenameModel(old_name="Donation", new_name="SingleDonation"),
    ]



from django.urls import path

from wagtaildonate.api.views.configuration import DonationsConfigurationAPIView
from wagtaildonate.api.views.donations import DonationsCreateAPIView

app_name = "api"

urlpatterns = [
    path("donations/", DonationsCreateAPIView.as_view(), name="donations-create"),
    path(
        "configuration/donations/",
        DonationsConfigurationAPIView.as_view(),
        name="configuration-donations",
    ),
]



import os

__version__ = "1.9.5"


def get_version():
    dev_tag = os.environ.get("WAGTAIL_DONATE_DEV_TAG")
    if not dev_tag:
        return __version__
    try:
        dev_tag = int(dev_tag)
    except (ValueError, TypeError):
        raise ValueError("Dev version tag has to be a number.")
    return f"{__version__}.dev{dev_tag}"


default_app_config = "wagtaildonate.apps.WagtailDonateConfig"



from typing import Optional

from django.core.exceptions import ImproperlyConfigured

import braintree

from wagtaildonate.api.serializers.braintree import (
    BraintreePayInSerializer,
    BraintreeSingleDonationSerializer,
)
from wagtaildonate.exceptions import TransactionNotFound
from wagtaildonate.payment_methods.base import PaymentMethod
from wagtaildonate.transaction_status import TransactionStatus
from wagtaildonate.utils.braintree import map_status


class BraintreeMixin:
    serializer_classes = {
        "single": BraintreeSingleDonationSerializer,
        "payin": BraintreePayInSerializer,
    }
    assets = {
        "js": [
            {"src": "https://js.braintreegateway.com/web/3.76.4/js/client.min.js"},
            {
                "src": "https://js.braintreegateway.com/web/3.76.4/js/data-collector.min.js"
            },
        ]
    }

    def get_frontend_options(self):
        return {
            "client_authorization": self.get_client_token(),
        }

    def get_config(self):
        merchant_id = self.options.get("MERCHANT_ID", None)
        public_key = self.options.get("PUBLIC_KEY", None)
        private_key = self.options.get("PRIVATE_KEY", None)

        if None in [merchant_id, public_key, private_key]:
            raise ImproperlyConfigured(
                "One or more of these options have not been defined: "
                "MERCHANT_ID, PUBLIC_KEY, PRIVATE_KEY"
            )
        if self.options.get("SANDBOX", False):
            environment = braintree.Environment.Sandbox
        else:
            environment = braintree.Environment.Production
        return merchant_id, public_key, private_key, environment

    def get_gateway(self):
        merchant_id, public_key, private_key, environment = self.get_config()
        return braintree.BraintreeGateway(
            braintree.Configuration(
                environment,
                merchant_id=merchant_id,
                public_key=public_key,
                private_key=private_key,
            )
        )

    def get_transaction_status_for_transaction(self, transaction_id, gateway=None):
        if gateway is None:
            gateway = self.get_gateway()
        try:
            transaction = gateway.transaction.find(transaction_id)
        except braintree.exceptions.not_found_error.NotFoundError as e:
            raise TransactionNotFound from e
        return TransactionStatus(
            provider_transaction_status=transaction.status,
            transaction_status=map_status(transaction.status),
        )

    def get_client_token(self):
        return self.get_gateway().client_token.generate(
            {"merchant_account_id": self.get_merchant_account_id()}
        )

    def get_merchant_account_id(self) -> Optional[str]:
        merchant_account_id = self.context.get("braintree_merchant_account_id")
        if merchant_account_id:
            return merchant_account_id

        return self.options.get("MERCHANT_ACCOUNT_ID")

    def should_send_postal_code(self) -> bool:
        return self.options.get("SEND_POSTAL_CODE", False)

    def should_send_country_code(self) -> bool:
        return self.options.get("SEND_COUNTRY_CODE", False)

    def should_send_street_address(self) -> bool:
        return self.options.get("SEND_STREET_ADDRESS", False)

    def should_send_name(self) -> bool:
        return self.options.get("SEND_NAME", False)

    def should_send_town(self) -> bool:
        return self.options.get("SEND_TOWN", False)

    def should_send_email_address(self) -> bool:
        return self.options.get("SEND_EMAIL_ADDRESS", False)

    def should_send_phone_number(self) -> bool:
        return self.options.get("SEND_PHONE_NUMBER", False)


class BraintreeCreditCardPaymentMethod(BraintreeMixin, PaymentMethod):
    code = "braintree_credit_card"
    assets = {
        **BraintreeMixin.assets,
        "js": BraintreeMixin.assets.get("js", [])
        + [
            {
                "src": "https://js.braintreegateway.com/web/3.76.4/js/hosted-fields.min.js"
            },
            {
                "src": "https://js.braintreegateway.com/web/3.76.4/js/three-d-secure.min.js"
            },
        ],
    }

    def get_frontend_options(self):
        return {
            **super().get_frontend_options(),
            "send_cardholder_name": self.options.get("SEND_CARDHOLDER_NAME", False),
            "send_billing_address": self.options.get("SEND_BILLING_ADDRESS", False),
            "three_d_secure_enabled": self.options.get("THREE_D_SECURE_ENABLED", False),
        }


class BraintreePayPalPaymentMethod(BraintreeMixin, PaymentMethod):
    code = "braintree_paypal"
    assets = {
        **BraintreeMixin.assets,
        "js": BraintreeMixin.assets.get("js", [])
        + [
            {"src": "https://www.paypalobjects.com/api/checkout.js"},
            {
                "src": "https://js.braintreegateway.com/web/3.76.4/js/paypal-checkout.min.js"
            },
        ],
    }

    def get_frontend_options(self):
        return {
            **super().get_frontend_options(),
            "sandbox": self.options.get("SANDBOX", False),
            "currency": self.options.get("CURRENCY", "USD"),
            "auto_fill_shipping_address": self.options.get(
                "AUTOFILL_SHIPPING_ADDRESS", False
            ),
            "auto_fill_phone_number": self.options.get("AUTOFILL_PHONE_NUMBER", False),
            "auto_fill_name": self.options.get("AUTOFILL_NAME", False),
            "auto_fill_email_address": self.options.get(
                "AUTOFILL_EMAIL_ADDRESS", False
            ),
            "locale": self.options.get("LOCALE"),
            "display_name": self.options.get("DISPLAY_NAME"),
            # PayPal checkout button settings
            # https://developer.paypal.com/docs/archive/checkout/how-to/customize-button/
            "tagline": self.options.get("BUTTON_TAGLINE"),
            "label": self.options.get("BUTTON_LABEL"),
            "size": self.options.get("BUTTON_SIZE"),
            "shape": self.options.get("BUTTON_SHAPE"),
            "height": self.options.get("BUTTON_HEIGHT"),
            "color": self.options.get("BUTTON_COLOR"),
        }


class BraintreeApplePayPaymentMethod(BraintreeMixin, PaymentMethod):
    code = "braintree_apple_pay"
    assets = {
        **BraintreeMixin.assets,
        "js": BraintreeMixin.assets.get("js", [])
        + [{"src": "https://js.braintreegateway.com/web/3.76.4/js/apple-pay.min.js"}],
    }

    def get_frontend_options(self):
        return {
            **super().get_frontend_options(),
            "auto_fill_billing_address": self.options.get(
                "AUTOFILL_BILLING_ADDRESS", False
            ),
            "auto_fill_phone_number": self.options.get("AUTOFILL_PHONE_NUMBER", False),
            "auto_fill_name": self.options.get("AUTOFILL_NAME", False),
            "auto_fill_email_address": self.options.get(
                "AUTOFILL_EMAIL_ADDRESS", False
            ),
            "display_name": self.options.get("DISPLAY_NAME", "Donations"),
            "line_item_label": self.options.get("LINE_ITEM_LABEL", "Donation"),
        }


class BraintreeGooglePayPaymentMethod(BraintreeMixin, PaymentMethod):
    code = "braintree_google_pay"
    assets = {
        **BraintreeMixin.assets,
        "js": BraintreeMixin.assets.get("js", [])
        + [
            {"src": "https://pay.google.com/gp/p/js/pay.js"},
            {
                "src": "https://js.braintreegateway.com/web/3.76.4/js/google-payment.min.js"
            },
        ],
    }

    def get_frontend_options(self):
        return {
            **super().get_frontend_options(),
            "sandbox": self.options.get("SANDBOX", False),
            "currency": self.options.get("CURRENCY", "USD"),
            "google_merchant_id": self.options.get("GOOGLE_MERCHANT_ID"),
            "auto_fill_billing_address": self.options.get(
                "AUTOFILL_BILLING_ADDRESS", False
            ),
            "auto_fill_phone_number": self.options.get("AUTOFILL_PHONE_NUMBER", False),
            "auto_fill_name": self.options.get("AUTOFILL_NAME", False),
            "auto_fill_email_address": self.options.get(
                "AUTOFILL_EMAIL_ADDRESS", False
            ),
            # Google Pay button settings
            # https://developers.google.com/pay/api/web/reference/request-objects#ButtonOptions
            "button_type": self.options.get("BUTTON_TYPE"),
            "button_color": self.options.get("BUTTON_COLOR"),
        }



from typing import Type

from rest_framework import serializers


def get_single_donation_serializer_string() -> str:
    from wagtaildonate.settings import donate_settings

    return donate_settings.SINGLE_DONATION_SERIALIZER_CLASS


def get_single_donation_serializer_class() -> Type[serializers.Serializer]:
    from django.utils.module_loading import import_string

    return import_string(get_single_donation_serializer_string())


def get_recurring_donation_serializer_string() -> str:
    from wagtaildonate.settings import donate_settings

    return donate_settings.RECURRING_DONATION_SERIALIZER_CLASS


def get_recurring_donation_serializer_class() -> Type[serializers.Serializer]:
    from django.utils.module_loading import import_string

    return import_string(get_recurring_donation_serializer_string())


def get_pay_in_serializer_string() -> str:
    from wagtaildonate.settings import donate_settings

    return donate_settings.PAY_IN_SERIALIZER_CLASS


def get_pay_in_serializer_class() -> Type[serializers.Serializer]:
    from django.utils.module_loading import import_string

    return import_string(get_pay_in_serializer_string())


def get_pay_in_event_serializer_string() -> str:
    from wagtaildonate.settings import donate_settings

    return donate_settings.PAY_IN_EVENT_SERIALIZER_CLASS


def get_pay_in_event_serializer_class() -> Type[serializers.Serializer]:
    from django.utils.module_loading import import_string

    return import_string(get_pay_in_event_serializer_string())


def get_configuration_serializer_string() -> str:
    from wagtaildonate.settings import donate_settings

    return donate_settings.CONFIGURATION_SERIALIZER_CLASS


def get_configuration_serializer_class() -> Type[serializers.Serializer]:
    from django.utils.module_loading import import_string

    return import_string(get_configuration_serializer_string())



from django.db import models


class DonationQuerySet(models.QuerySet):
    def settling(self):
        return self.filter(transaction_status=self.model.STATUS_SETTLING)

    def failed(self):
        return self.filter(transaction_status=self.model.STATUS_FAILED)

    def settled(self):
        return self.filter(transaction_status=self.model.STATUS_SETTLED)



# Generated by Django 2.2.11 on 2020-03-11 11:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0007_country"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="gift_aid_declaration",
            field=models.BooleanField(default=False, verbose_name="Gift Aid"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="donation",
            name="on_behalf_of_organisation",
            field=models.BooleanField(
                default=False, verbose_name="On behalf of organisation"
            ),
            preserve_default=False,
        ),
    ]



import django.db.models.deletion
from django.db import migrations, models

import modelcluster.fields
from wagtail.images import get_image_model_string

image_model = get_image_model_string()


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailcore", "0041_group_collection_permissions_verbose_name_plural"),
        migrations.swappable_dependency(get_image_model_string()),
    ]

    operations = [
        migrations.CreateModel(
            name="DonationCampaignPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.Page",
                    ),
                ),
                ("introduction", models.TextField(blank=True)),
                ("hero_summary_text", models.TextField(blank=True)),
                ("allow_single_donations", models.BooleanField(default=False)),
                ("allow_regular_donations", models.BooleanField(default=False)),
                (
                    "show_other_amount_option",
                    models.BooleanField(
                        default=False,
                        help_text="Offer the user the option to specify a custom amount.",
                    ),
                ),
                (
                    "default_donation_frequency",
                    models.CharField(
                        choices=[("single", "single"), ("regular", "regular")],
                        default="single",
                        help_text="Default donation frequency to prompt the user with. Only applies if both weekly and monthly donations are allowed.",
                        max_length=20,
                    ),
                ),
                (
                    "hero_image",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=image_model,
                    ),
                ),
            ],
            options={"abstract": False},
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="DonationFundraisingPayInPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.Page",
                    ),
                ),
            ],
            options={"abstract": False},
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="DonationPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.Page",
                    ),
                ),
                ("introduction", models.TextField(blank=True)),
                ("hero_summary_text", models.TextField(blank=True)),
                ("allow_single_donations", models.BooleanField(default=False)),
                ("allow_regular_donations", models.BooleanField(default=False)),
                (
                    "show_other_amount_option",
                    models.BooleanField(
                        default=False,
                        help_text="Offer the user the option to specify a custom amount.",
                    ),
                ),
                (
                    "default_donation_frequency",
                    models.CharField(
                        choices=[("single", "single"), ("regular", "regular")],
                        default="single",
                        help_text="Default donation frequency to prompt the user with. Only applies if both weekly and monthly donations are allowed.",
                        max_length=20,
                    ),
                ),
                (
                    "hero_image",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=image_model,
                    ),
                ),
            ],
            options={"abstract": False},
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="SuggestedDonationDonatePagePlacementSingle",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sort_order",
                    models.IntegerField(blank=True, editable=False, null=True),
                ),
                ("amount", models.IntegerField()),
                (
                    "page",
                    modelcluster.fields.ParentalKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="suggested_amounts_donate_page_single",
                        to="wagtaildonate.DonationPage",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="SuggestedDonationDonatePagePlacementRegular",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sort_order",
                    models.IntegerField(blank=True, editable=False, null=True),
                ),
                ("amount", models.IntegerField()),
                (
                    "page",
                    modelcluster.fields.ParentalKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="suggested_amounts_donate_page_regular",
                        to="wagtaildonate.DonationPage",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="SuggestedDonationCampaignPagePlacementSingle",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sort_order",
                    models.IntegerField(blank=True, editable=False, null=True),
                ),
                ("amount", models.IntegerField()),
                (
                    "page",
                    modelcluster.fields.ParentalKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="suggested_amounts_campaign_page_single",
                        to="wagtaildonate.DonationCampaignPage",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="SuggestedDonationCampaignPagePlacementRegular",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sort_order",
                    models.IntegerField(blank=True, editable=False, null=True),
                ),
                ("amount", models.IntegerField()),
                (
                    "page",
                    modelcluster.fields.ParentalKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="suggested_amounts_campaign_page_regular",
                        to="wagtaildonate.DonationCampaignPage",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
    ]



# Generated by Django 2.2.12 on 2020-08-25 13:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0031_remove_conrete_models"),
    ]

    operations = [
        migrations.CreateModel(
            name="PayInEvent",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("event_code", models.CharField(max_length=50, unique=True)),
                ("event_name", models.CharField(max_length=255)),
                ("fundraiser_reference_required", models.BooleanField()),
            ],
            options={
                "verbose_name": "Pay In Event",
                "verbose_name_plural": "Pay In Events",
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="payin",
            name="event_code",
            field=models.CharField(blank=True, max_length=50),
        ),
        migrations.AddField(
            model_name="payin",
            name="event_name",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="payin",
            name="fundraiser_reference",
            field=models.CharField(blank=True, max_length=50),
        ),
    ]



# Generated by Django 2.2.7 on 2020-02-28 11:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0003_donation"),
    ]

    operations = [
        migrations.AddField(
            model_name="donationcampaignpage",
            name="thank_you_strapline",
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name="donationfundraisingpayinpage",
            name="thank_you_strapline",
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name="donationpage",
            name="thank_you_strapline",
            field=models.TextField(blank=True),
        ),
        migrations.AlterField(
            model_name="donationcampaignpage",
            name="default_donation_frequency",
            field=models.CharField(
                choices=[("single", "single"), ("regular", "regular")],
                default="single",
                help_text="Default donation frequency to prompt the user with. Only applies if both single and regular donations are allowed.",
                max_length=20,
            ),
        ),
        migrations.AlterField(
            model_name="donationpage",
            name="default_donation_frequency",
            field=models.CharField(
                choices=[("single", "single"), ("regular", "regular")],
                default="single",
                help_text="Default donation frequency to prompt the user with. Only applies if both single and regular donations are allowed.",
                max_length=20,
            ),
        ),
    ]



# Generated by Django 2.2.7 on 2020-03-26 15:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0009_donation_memory"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="payment_method",
            field=models.CharField(default="", max_length=100),
            preserve_default=False,
        ),
    ]



from wagtaildonate.address_lookups.base import AddressLookup


class LoqateAddressLookup(AddressLookup):
    code = "loqate"
    assets = {
        "js": [{"src": "https://services.postcodeanywhere.co.uk/js/address-3.91.js"}],
        "css": [
            {
                "href": "https://services.postcodeanywhere.co.uk/css/address-3.91.css",
                "rel": "stylesheet",
                "type": "text/css",
            },
        ],
    }

    def get_frontend_options(self):
        return {
            "api_key": self.options["API_KEY"],
            "set_country_by_ip": self.options.get("SET_COUNTRY_BY_IP", True),
        }



from decimal import Decimal

from django.test import TestCase, override_settings

import rest_framework

from wagtaildonate.api.serializers.braintree import BraintreeSingleDonationSerializer
from wagtaildonate.api.serializers.donations import (
    PayInSerializer,
    RecurringDonationSerializer,
    SingleDonationSerializer,
)
from wagtaildonate.api.serializers.payment_method import (
    PaymentMethodAndFrequencySerializer,
)
from wagtaildonate.models import PayIn, PayInEvent, RecurringDonation, SingleDonation


@override_settings(
    WAGTAIL_DONATE={
        "PAYMENT_METHODS": [
            {
                "class": "wagtaildonate.payment_methods.braintree.BraintreePayPalPaymentMethod",
                "options": {
                    "SANDBOX": True,
                    "MERCHANT_ID": "--------------",
                    "MERCHANT_ACCOUNT_ID": "---",
                    "PUBLIC_KEY": "---",
                    "PRIVATE_KEY": "----",
                    "CURRENCY": "GBP",
                    "BUTTON_TAGLINE": "false",
                    "BUTTON_LABEL": "paypal",
                    "BUTTON_SIZE": "responsive",
                    "BUTTON_HEIGHT": 50,
                    "BUTTON_SHAPE": "rect",
                    "AUTOFILL_SHIPPING_ADDRESS": True,
                    "AUTOFILL_PHONE_NUMBER": True,
                    "AUTOFILL_NAME": True,
                    "AUTOFILL_EMAIL_ADDRESS": True,
                    "DISPLAY_NAME": "Wagtail Donate",
                    "LOCALE": "en_GB",
                },
            },
        ],
    }
)
class TestPaymentMethodAndFrequencySerializer(TestCase):
    def test_payment_method_required(self):
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            PaymentMethodAndFrequencySerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("payment_method", detail)
        self.assertTrue(len(detail["payment_method"]), 1)
        self.assertEqual(detail["payment_method"][0].code, "required")

    def test_frequency_required(self):
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            PaymentMethodAndFrequencySerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("frequency", detail)
        self.assertTrue(len(detail["frequency"]), 1)
        self.assertEqual(detail["frequency"][0].code, "required")

    def test_invalid_payment_method_not_accepted(self):
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            PaymentMethodAndFrequencySerializer(
                data={"payment_method": "some_payment_method", "frequency": "single"}
            ).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("payment_method", detail)
        self.assertTrue(len(detail["payment_method"]), 1)
        self.assertEqual(detail["payment_method"][0].code, "invalid")

    def test_valid_payment_method_and_frequency_accepted(self):
        from wagtaildonate.payment_methods.braintree import BraintreePayPalPaymentMethod

        serializer = PaymentMethodAndFrequencySerializer(
            data={"payment_method": "braintree_paypal", "frequency": "single"}
        )
        serializer.is_valid(raise_exception=True)
        self.assertIsInstance(
            serializer.validated_data["payment_method"], BraintreePayPalPaymentMethod
        )

    def test_valid_payment_method_and_invalid_frequency_not_accepted(self):
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            serializer = PaymentMethodAndFrequencySerializer(
                data={"payment_method": "braintree_paypal", "frequency": "monthly"}
            )
            serializer.is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("frequency", detail)
        self.assertTrue(len(detail["frequency"]), 1)
        self.assertEqual(detail["frequency"][0].code, "invalid")
        self.assertEqual(
            str(detail["frequency"][0]),
            "The frequency is not supported by the payment method.",
        )


class TestSingleDonationSerializer(TestCase):
    def test_amount_required(self):
        """Assert that the amount field is required"""
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            SingleDonationSerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("amount", detail)
        self.assertTrue(len(detail["amount"]), 1)
        self.assertEqual(detail["amount"][0].code, "required")

    def test_payment_method_required(self):
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            SingleDonationSerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("payment_method", detail)
        self.assertTrue(len(detail["payment_method"]), 1)
        self.assertEqual(detail["payment_method"][0].code, "required")

    def test_phone_invalid_not_accepted(self):
        """Assert that invalid phone numbers are not accepted"""
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            SingleDonationSerializer(
                data={"phone_number": "i am not a phone number"}
            ).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("phone_number", detail)
        self.assertTrue(len(detail["phone_number"]), 1)
        self.assertEqual(detail["phone_number"][0].code, "invalid")

    def test_phone_international_accepted(self):
        """
        Assert that a valid international phone number does not
        raise any validation errors regarding the phone number.
        """
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            SingleDonationSerializer(
                data={"phone_number": "+44 7771234567"}  # Fake international UK number
            ).is_valid(raise_exception=True)
        self.assertNotIn("phone_number", cm.exception.detail)

    @override_settings(PHONENUMBER_DEFAULT_REGION="GB")
    def test_phone_national_accepted(self):
        """
        Assert that a valid national phone number does not
        raise any validation errors regarding the phone number.
        """
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            SingleDonationSerializer(
                data={"phone_number": "07771234567"}  # Fake national UK number
            ).is_valid(raise_exception=True)
        self.assertNotIn("phone_number", cm.exception.detail)

    def test_create_no_giftaid(self):
        """Assert that a Donation can be created from a serializer"""
        serializer = SingleDonationSerializer(
            data={
                "payment_method": "dummy_payment_method",
                "transaction_id": "test_id",
                "amount": 50,
                "first_name": "Alice",
                "surname": "Johnson",
                "email": "ajohnson@example.com",
                "phone_number": "+44 7771234567",  # Fake international UK number
                "address_line_1": "2nd floor",
                "address_line_2": "123 Test Street",
                "town": "Test Town",
                "postal_code": "TST1 1AB",
                "country": "GB",
                "on_behalf_of_organisation": False,
                "gift_aid_declaration": False,
                "in_memory": False,
                "in_memory_of": "",
            }
        )
        self.assertTrue(serializer.is_valid(raise_exception=True))
        donation = serializer.save()
        self.assertTrue(SingleDonation.objects.filter(id=donation.id).exists())

    def test_default_minimum_amount_fails(self):
        serializer = SingleDonationSerializer(
            data={
                "payment_method": "dummy_payment_method",
                "transaction_id": "test_id",
                "amount": "0.99",
                "first_name": "Alice",
                "surname": "Johnson",
                "email": "ajohnson@example.com",
                "phone_number": "+44 7771234567",  # Fake international UK number
                "address_line_1": "2nd floor",
                "address_line_2": "123 Test Street",
                "town": "Test Town",
                "postal_code": "TST1 1AB",
                "country": "GB",
                "on_behalf_of_organisation": False,
                "gift_aid_declaration": False,
                "in_memory": False,
                "in_memory_of": "",
            }
        )
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            serializer.is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("amount", detail)
        self.assertTrue(len(detail["amount"]), 1)
        self.assertEqual(detail["amount"][0].code, "invalid")
        self.assertEqual(str(detail["amount"][0]), "Minimum amount is 1.00.")

    def test_default_minimum_amount_passes(self):
        serializer = SingleDonationSerializer(
            data={
                "payment_method": "dummy_payment_method",
                "transaction_id": "test_id",
                "amount": "1.00",
                "first_name": "Alice",
                "surname": "Johnson",
                "email": "ajohnson@example.com",
                "phone_number": "+44 7771234567",  # Fake international UK number
                "address_line_1": "2nd floor",
                "address_line_2": "123 Test Street",
                "town": "Test Town",
                "postal_code": "TST1 1AB",
                "country": "GB",
                "on_behalf_of_organisation": False,
                "gift_aid_declaration": False,
                "in_memory": False,
                "in_memory_of": "",
            }
        )
        serializer.is_valid(raise_exception=True)
        donation = serializer.save()
        self.assertEqual(donation.amount, Decimal("1.00"))

    def test_default_maximum_amount_fails(self):
        serializer = SingleDonationSerializer(
            data={
                "payment_method": "dummy_payment_method",
                "transaction_id": "test_id",
                "amount": "10000.01",
                "first_name": "Alice",
                "surname": "Johnson",
                "email": "ajohnson@example.com",
                "phone_number": "+44 7771234567",  # Fake international UK number
                "address_line_1": "2nd floor",
                "address_line_2": "123 Test Street",
                "town": "Test Town",
                "postal_code": "TST1 1AB",
                "country": "GB",
                "on_behalf_of_organisation": False,
                "gift_aid_declaration": False,
                "in_memory": False,
                "in_memory_of": "",
            }
        )
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            serializer.is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("amount", detail)
        self.assertTrue(len(detail["amount"]), 1)
        self.assertEqual(detail["amount"][0].code, "invalid")
        self.assertEqual(str(detail["amount"][0]), "Maximum amount is 10000.00.")

    def test_default_maximum_amount_passes(self):
        serializer = SingleDonationSerializer(
            data={
                "payment_method": "dummy_payment_method",
                "transaction_id": "test_id",
                "amount": "10000.00",
                "first_name": "Alice",
                "surname": "Johnson",
                "email": "ajohnson@example.com",
                "phone_number": "+44 7771234567",  # Fake international UK number
                "address_line_1": "2nd floor",
                "address_line_2": "123 Test Street",
                "town": "Test Town",
                "postal_code": "TST1 1AB",
                "country": "GB",
                "on_behalf_of_organisation": False,
                "gift_aid_declaration": False,
                "in_memory": False,
                "in_memory_of": "",
            }
        )
        serializer.is_valid(raise_exception=True)
        donation = serializer.save()
        self.assertEqual(donation.amount, Decimal("10000.00"))

    def test_custom_minimum_amount_fails(self):
        serializer = SingleDonationSerializer(
            context={"minimum_amount": Decimal("1.50")},
            data={
                "payment_method": "dummy_payment_method",
                "transaction_id": "test_id",
                "amount": "1.49",
                "first_name": "Alice",
                "surname": "Johnson",
                "email": "ajohnson@example.com",
                "phone_number": "+44 7771234567",  # Fake international UK number
                "address_line_1": "2nd floor",
                "address_line_2": "123 Test Street",
                "town": "Test Town",
                "postal_code": "TST1 1AB",
                "country": "GB",
                "on_behalf_of_organisation": False,
                "gift_aid_declaration": False,
                "in_memory": False,
                "in_memory_of": "",
            },
        )
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            serializer.is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("amount", detail)
        self.assertTrue(len(detail["amount"]), 1)
        self.assertEqual(detail["amount"][0].code, "invalid")
        self.assertEqual(str(detail["amount"][0]), "Minimum amount is 1.50.")

    def test_custom_maximum_amount_fails(self):
        serializer = SingleDonationSerializer(
            context={"maximum_amount": Decimal("3.05")},
            data={
                "payment_method": "dummy_payment_method",
                "transaction_id": "test_id",
                "amount": "3.06",
                "first_name": "Alice",
                "surname": "Johnson",
                "email": "ajohnson@example.com",
                "phone_number": "+44 7771234567",  # Fake international UK number
                "address_line_1": "2nd floor",
                "address_line_2": "123 Test Street",
                "town": "Test Town",
                "postal_code": "TST1 1AB",
                "country": "GB",
                "on_behalf_of_organisation": False,
                "gift_aid_declaration": False,
                "in_memory": False,
                "in_memory_of": "",
            },
        )
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            serializer.is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("amount", detail)
        self.assertTrue(len(detail["amount"]), 1)
        self.assertEqual(detail["amount"][0].code, "invalid")
        self.assertEqual(str(detail["amount"][0]), "Maximum amount is 3.05.")


class TestBraintreeSingleDonationSerializer(TestCase):
    def test_payment_method_nonce_requred(self):
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            BraintreeSingleDonationSerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("payment_method_nonce", detail)
        self.assertTrue(len(detail["payment_method_nonce"]), 1)
        self.assertEqual(detail["payment_method_nonce"][0].code, "required")

    def test_amount_required(self):
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            BraintreeSingleDonationSerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("amount", detail)
        self.assertTrue(len(detail["amount"]), 1)
        self.assertEqual(detail["amount"][0].code, "required")


class TestRecurringDonationSerializer(TestCase):
    def test_payment_method_required(self):
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            SingleDonationSerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("payment_method", detail)
        self.assertTrue(len(detail["payment_method"]), 1)
        self.assertEqual(detail["payment_method"][0].code, "required")

    def test_amount_required(self):
        """Assert that the amount field is required"""
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            RecurringDonationSerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("amount", detail)
        self.assertTrue(len(detail["amount"]), 1)
        self.assertEqual(detail["amount"][0].code, "required")

    def test_frequency_required(self):
        """Assert that the frequency field is required"""
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            RecurringDonationSerializer(data={}).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("frequency", detail)
        self.assertTrue(len(detail["frequency"]), 1)
        self.assertEqual(detail["frequency"][0].code, "required")

    def test_phone_invalid_not_accepted(self):
        """Assert that invalid phone numbers are not accepted"""
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            RecurringDonationSerializer(
                data={"phone_number": "i am not a phone number"}
            ).is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("phone_number", detail)
        self.assertTrue(len(detail["phone_number"]), 1)
        self.assertEqual(detail["phone_number"][0].code, "invalid")

    def test_phone_international_accepted(self):
        """
        Assert that a valid international phone number does not
        raise any validation errors regarding the phone number.
        """
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            RecurringDonationSerializer(
                data={"phone_number": "+44 7771234567"}  # Fake international UK number
            ).is_valid(raise_exception=True)
        self.assertNotIn("phone_number", cm.exception.detail)

    @override_settings(PHONENUMBER_DEFAULT_REGION="GB")
    def test_phone_national_accepted(self):
        """
        Assert that a valid national phone number does not
        raise any validation errors regarding the phone number.
        """
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            RecurringDonationSerializer(
                data={"phone_number": "07771234567"}  # Fake national UK number
            ).is_valid(raise_exception=True)
        self.assertNotIn("phone_number", cm.exception.detail)

    def test_create_no_giftaid(self):
        """Assert that a Donation can be created from a serializer"""
        serializer = RecurringDonationSerializer(
            data={
                "payment_method": "dummy_payment_method",
                "transaction_id": "test_id",
                "amount": 50,
                "frequency": "monthly",
                "first_name": "Alice",
                "surname": "Johnson",
                "email": "ajohnson@example.com",
                "phone_number": "+44 7771234567",  # Fake international UK number
                "address_line_1": "2nd floor",
                "address_line_2": "123 Test Street",
                "town": "Test Town",
                "postal_code": "TST1 1AB",
                "country": "GB",
                "on_behalf_of_organisation": False,
                "gift_aid_declaration": False,
                "in_memory": False,
                "in_memory_of": "",
            }
        )
        self.assertTrue(serializer.is_valid(raise_exception=True))
        donation = serializer.save()
        self.assertTrue(RecurringDonation.objects.filter(id=donation.id).exists())


base_pay_in_data = {
    "payment_method": "dummy_payment_method",
    "transaction_id": "test_id",
    "amount": 50,
    "frequency": "single",
    "first_name": "Alice",
    "surname": "Johnson",
    "email": "ajohnson@example.com",
    "phone_number": "+44 7771234567",  # Fake international UK number
    "address_line_1": "2nd floor",
    "address_line_2": "123 Test Street",
    "town": "Test Town",
    "postal_code": "TST1 1AB",
    "country": "GB",
    "on_behalf_of_organisation": False,
    "in_memory": False,
    "in_memory_of": "",
}


class TestPayInSerializer(TestCase):
    def setUp(self):
        self.pay_in_event_1 = PayInEvent.objects.create(
            event_code="evt1",
            event_name="Event 1",
            fundraiser_reference_required=False,
        )
        self.pay_in_event_2 = PayInEvent.objects.create(
            event_code="evt2",
            event_name="Event 2",
            fundraiser_reference_required=True,
        )

    @override_settings(WAGTAIL_DONATE={"PAY_IN_EVENTS_ENABLED": False})
    def test_create_with_pay_in_events_disabled(self):
        serializer = PayInSerializer(data=base_pay_in_data)
        self.assertTrue(serializer.is_valid(raise_exception=True))
        donation = serializer.save()
        self.assertTrue(PayIn.objects.filter(id=donation.id).exists())

    def test_create_with_pay_in_event_doesnt_require_urn(self):
        serializer = PayInSerializer(
            data={
                **base_pay_in_data,
                "event_code": self.pay_in_event_1.event_code,
                "event_name": self.pay_in_event_1.event_name,
            }
        )
        self.assertTrue(serializer.is_valid(raise_exception=True))
        donation = serializer.save()
        self.assertTrue(PayIn.objects.filter(id=donation.id).exists())
        self.assertEqual(donation.event_code, self.pay_in_event_1.event_code)

    def test_create_with_pay_in_event_requires_fundraiser_reference(self):
        # create should fail if fundraiser reference is not provided
        serializer = PayInSerializer(
            data={
                **base_pay_in_data,
                "event_code": self.pay_in_event_2.event_code,
                "event_name": self.pay_in_event_2.event_name,
            }
        )
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            serializer.is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("fundraiser_reference", detail)
        self.assertEqual(
            str(detail["fundraiser_reference"][0]),
            "Fundraiser reference must be entered.",
        )

        # create should succeed and donation created if fundraiser reference is provided
        serializer = PayInSerializer(
            data={
                **base_pay_in_data,
                "event_code": self.pay_in_event_2.event_code,
                "event_name": self.pay_in_event_2.event_name,
                "fundraiser_reference": "abcd1234",
            }
        )
        self.assertTrue(serializer.is_valid(raise_exception=True))
        donation = serializer.save()
        self.assertTrue(PayIn.objects.filter(id=donation.id).exists())
        self.assertEqual(donation.event_code, self.pay_in_event_2.event_code)

    @override_settings(
        WAGTAIL_DONATE={"PAY_IN_OTHER_EVENT_REQUIRE_FUNDRAISER_REFERENCE": False}
    )
    def test_create_with_other_pay_in_event_doesnt_require_fundraiser_reference(self):
        serializer = PayInSerializer(
            data={**base_pay_in_data, "event_code": "other", "event_name": "test name"}
        )
        self.assertTrue(serializer.is_valid(raise_exception=True))
        donation = serializer.save()
        self.assertTrue(PayIn.objects.filter(id=donation.id).exists())
        self.assertEqual(donation.event_code, "other")

    def test_create_with_other_pay_in_event_requires_fundraiser_reference(self):
        # create should fail if fundraiser reference is not provided
        serializer = PayInSerializer(
            data={**base_pay_in_data, "event_code": "other", "event_name": "test name"}
        )
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            serializer.is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("fundraiser_reference", detail)
        self.assertEqual(
            str(detail["fundraiser_reference"][0]),
            "Fundraiser reference must be entered.",
        )

        # create should succeed and donation created if fundraiser reference is provided
        serializer = PayInSerializer(
            data={
                **base_pay_in_data,
                "event_code": "other",
                "event_name": "test name",
                "fundraiser_reference": "abcd1234",
            }
        )
        self.assertTrue(serializer.is_valid(raise_exception=True))
        donation = serializer.save()
        self.assertTrue(PayIn.objects.filter(id=donation.id).exists())
        self.assertEqual(donation.event_code, "other")

    def test_create_with_invalid_pay_in_event(self):
        serializer = PayInSerializer(
            data={**base_pay_in_data, "event_code": "invalid_event_code"}
        )
        with self.assertRaises(rest_framework.exceptions.ValidationError) as cm:
            serializer.is_valid(raise_exception=True)
        detail = cm.exception.detail
        self.assertIn("event_code", detail)
        self.assertEqual(
            str(detail["event_code"][0]),
            "Pay-in event does not exist. Please select a valid event.",
        )



# Generated by Django 2.2.12 on 2020-05-27 10:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0024_donationexportlog_default_permissions"),
    ]

    operations = [
        migrations.AddField(
            model_name="donationexportlog",
            name="remote_addr",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="REMOTE_ADDR"
            ),
        ),
        migrations.AddField(
            model_name="donationexportlog",
            name="remote_host",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="REMOTE_HOST"
            ),
        ),
        migrations.AddField(
            model_name="donationexportlog",
            name="user_agent",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="donationexportlog",
            name="x_forwarded_for",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="X-Forwarded-For"
            ),
        ),
    ]



# Generated by Django 2.2.11 on 2020-04-09 10:40

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0013_donation_export_log"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="created_at",
            field=models.DateTimeField(
                auto_now_add=True,
                default=django.utils.timezone.now,
                verbose_name="Created at",
            ),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-22 01:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildonate", "0022_thank_you_page_content"),
    ]

    operations = [
        migrations.AddField(
            model_name="donation",
            name="email_consent",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="donation",
            name="phone_consent",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="donation",
            name="post_consent",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="donation",
            name="sms_consent",
            field=models.BooleanField(
                blank=True, default=None, null=True, verbose_name="SMS consent"
            ),
        ),
        migrations.AddField(
            model_name="recurringdonation",
            name="email_consent",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="recurringdonation",
            name="phone_consent",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="recurringdonation",
            name="post_consent",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="recurringdonation",
            name="sms_consent",
            field=models.BooleanField(
                blank=True, default=None, null=True, verbose_name="SMS consent"
            ),
        ),
    ]



from decimal import Decimal
from typing import ClassVar, List, Optional
from urllib.parse import urlsplit, urlunsplit

from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import QuerySet
from django.http import Http404, HttpResponse, HttpResponseRedirect, QueryDict
from django.urls import reverse
from django.views.generic.base import TemplateView

from wagtail.models import Page

from wagtaildonate.models import AbstractDonationLandingPage, AbstractPayInLandingPage
from wagtaildonate.models.utils import get_default_donation_page
from wagtaildonate.payment_methods.base import PaymentMethod
from wagtaildonate.settings import donate_settings

CAMPAIGN_PAGE_ID_PARAM = "campaign"


class AmountForm(forms.Form):
    amount = forms.DecimalField(decimal_places=2)


class FrequencyForm(forms.Form):
    frequency = forms.ChoiceField(choices=[])

    def __init__(self, *args, **kwargs):
        allowed_frequencies = kwargs.pop("allowed_frequencies")
        default_frequency = kwargs.pop("default_frequency", None)
        super().__init__(*args, **kwargs)
        self.fields["frequency"].choices = [
            [value] * 2 for value in allowed_frequencies
        ]
        if default_frequency:
            self.fields["frequency"].initial = default_frequency
            self.fields["frequency"].required = False

    def clean_frequency(self):
        """
        If no frequency provided but the default is set, return the default one.
        """
        data = self.cleaned_data["frequency"]
        initial = self["frequency"].initial
        if not data and initial:
            return initial
        return data


class CampaignPageForm(forms.Form):
    def __init__(self, *args, **kwargs):
        queryset = kwargs.pop("queryset")
        super().__init__(*args, **kwargs)
        self.fields[CAMPAIGN_PAGE_ID_PARAM] = forms.ModelChoiceField(queryset=queryset)


class BaseCheckoutView(TemplateView):
    campaign_page: Optional[Page]
    amount: Optional[int]
    frequency: Optional[str]
    allowed_frequencies: ClassVar[List[str]] = []

    def __init__(self, *args, **kwargs):
        self.campaign_page = None
        self.amount = None
        self.frequency = None
        super().__init__(*args, **kwargs)

    def dispatch(self, request, *args, **kwargs):
        campaign_page_response = self.validate_campaign_page()
        if campaign_page_response is not None:
            return campaign_page_response

        frequency_response = self.validate_frequency()
        if frequency_response is not None:
            return frequency_response

        amount_response = self.validate_amount()
        if amount_response is not None:
            return amount_response

        return super().dispatch(request, *args, **kwargs)

    def get_allowed_frequencies(self) -> List[str]:
        return self.allowed_frequencies.copy()

    def get_default_frequency(self) -> Optional[str]:
        allowed_frequencies = tuple(self.get_allowed_frequencies())
        if len(allowed_frequencies) == 1:
            return allowed_frequencies[0]
        return

    def get_campaign_page_queryset(self) -> QuerySet:
        return Page.objects.live().public().specific()

    def amount_within_allowed_range(self, amount) -> bool:
        minimum_amount = Decimal(
            donate_settings.MINIMUM_AMOUNT_PER_FREQUENCY.get(self.frequency)
        )
        maximum_amount = Decimal(
            donate_settings.MAXIMUM_AMOUNT_PER_FREQUENCY.get(self.frequency)
        )
        if amount < minimum_amount or amount > maximum_amount:
            return False
        return True

    def validate_amount(self) -> Optional[HttpResponse]:
        form = AmountForm(self.request.GET)
        if not form.is_valid() or not self.amount_within_allowed_range(
            form.cleaned_data["amount"]
        ):
            return self.redirect_to_campaign_page()
        self.amount = form.cleaned_data["amount"]

    def validate_frequency(self) -> Optional[HttpResponse]:
        form = FrequencyForm(
            self.request.GET,
            allowed_frequencies=self.get_allowed_frequencies(),
            default_frequency=self.get_default_frequency(),
        )
        if not form.is_valid():
            return self.redirect_to_campaign_page()
        self.frequency = form.cleaned_data["frequency"]

    def validate_campaign_page(self) -> Optional[HttpResponse]:
        form = CampaignPageForm(
            self.request.GET, queryset=self.get_campaign_page_queryset()
        )
        if not form.is_valid():
            return self.get_campaign_page_does_not_exist_response()
        self.campaign_page = form.cleaned_data[CAMPAIGN_PAGE_ID_PARAM]

    def get_campaign_page_does_not_exist_response(self) -> HttpResponse:
        raise Http404("Page does not exist.")

    def get_campaign_page_url(self) -> str:
        return self.campaign_page.get_url(self.request)

    def get_thank_you_page_url(self) -> str:
        if hasattr(self, "get_thank_you_page_url"):
            return self.campaign_page.get_thank_you_page_url(self.request)
        return self.get_campaign_page_url()

    def redirect_to_campaign_page(self) -> HttpResponseRedirect:
        return HttpResponseRedirect(self.get_campaign_page_url())

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            success_url=self.get_thank_you_page_url(),
            amount=self.amount,
            frequency=self.frequency,
            campaign_page=self.campaign_page,
            recaptcha_site_key=donate_settings.RECAPTCHA_PUBLIC_KEY,
            configuration_url=self.get_configuration_url(),
        )
        return context

    def get_configuration_url(self) -> str:
        return reverse("wagtaildonate:api:configuration-donations")


class DonateCheckoutView(BaseCheckoutView):
    template_name = "wagtaildonate/checkout.html"
    allowed_frequencies = [
        PaymentMethod.FREQUENCY_SINGLE,
        PaymentMethod.FREQUENCY_MONTHLY,
        PaymentMethod.FREQUENCY_PAYIN,
    ]

    def get_campaign_page_queryset(self) -> QuerySet:
        return (
            super()
            .get_campaign_page_queryset()
            .filter(
                Page.objects.type_q(AbstractDonationLandingPage)
                | Page.objects.type_q(AbstractPayInLandingPage)
            )
        )

    def get_default_campaign_page(self) -> Page:
        return get_default_donation_page()

    def get_campaign_page_does_not_exist_response(self) -> HttpResponse:
        try:
            default_campaign_page = self.get_default_campaign_page()
        except ObjectDoesNotExist:
            return super().get_campaign_page_does_not_exist_response()
        # Redirect to the checkout form with a right page ID if the page ID
        # provided by user is missing or invalid.
        split_url = list(urlsplit(self.request.get_full_path()))
        query_dict = QueryDict(split_url[3], mutable=True)
        query_dict[CAMPAIGN_PAGE_ID_PARAM] = default_campaign_page.pk
        split_url[3] = query_dict.urlencode()
        return HttpResponseRedirect(urlunsplit(split_url))



# Generated by Django 2.2.12 on 2020-05-28 01:37

from django.db import migrations

from wagtail import fields as wagtail_fields

import wagtaildonate.models.pages


class Migration(migrations.Migration):
    dependencies = [
        ("wagtaildonate", "0025_donationexportlog_user_info"),
    ]

    operations = [
        migrations.AlterField(
            model_name="donationcampaignpage",
            name="thank_you_email_content",
            field=wagtail_fields.StreamField(
                [("paragraph", wagtaildonate.models.pages.EmailParagraphBlock())],
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="donationfundraisingpayinpage",
            name="thank_you_email_content",
            field=wagtail_fields.StreamField(
                [("paragraph", wagtaildonate.models.pages.EmailParagraphBlock())],
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="donationpage",
            name="thank_you_email_content",
            field=wagtail_fields.StreamField(
                [("paragraph", wagtaildonate.models.pages.EmailParagraphBlock())],
                blank=True,
            ),
        ),
    ]



from xml.etree import ElementTree

from django.utils.safestring import mark_safe


def get_source_code_for_js_assets(assets):
    code = ""
    for asset in assets:
        script = ElementTree.Element("script", attrib=asset)
        code += ElementTree.tostring(script, encoding="unicode", method="html")
    return mark_safe(code)


def get_source_code_for_css_assets(assets):
    code = ""
    for asset in assets:
        link = ElementTree.Element("link", attrib=asset)
        code += ElementTree.tostring(link, encoding="unicode", method="html")
    return mark_safe(code)


def get_source_code_for_assets(assets):
    js_assets = assets.get("js", [])
    css_assets = assets.get("css", [])
    code = ""
    code += get_source_code_for_js_assets(js_assets)
    code += get_source_code_for_css_assets(css_assets)
    return mark_safe(code)



from django.db import models

from wagtail import blocks
from wagtail.admin.panels import FieldPanel
from wagtail.fields import RichTextField, StreamField
from wagtail.images import get_image_model_string
from wagtail.snippets.models import register_snippet

image_model = get_image_model_string()


@register_snippet
class ThankYouCTASnippet(models.Model):
    title = models.CharField(max_length=255)
    summary = RichTextField(blank=True, max_length=255)

    image = models.ForeignKey(
        image_model, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )

    link = StreamField(
        blocks.StreamBlock(
            [
                (
                    "external_link",
                    blocks.StructBlock(
                        [("url", blocks.URLBlock()), ("title", blocks.CharBlock())],
                        icon="link",
                    ),
                ),
                (
                    "internal_link",
                    blocks.StructBlock(
                        [
                            ("page", blocks.PageChooserBlock()),
                            ("title", blocks.CharBlock(required=False)),
                        ],
                        icon="link",
                    ),
                ),
            ],
            max_num=1,
            required=True,
        ),
        blank=True,
        use_json_field=True,
    )

    panels = [
        FieldPanel("title"),
        FieldPanel("summary"),
        FieldPanel("image"),
        FieldPanel("link"),
    ]

    def get_link_text(self):
        # Link is required, so we should always have
        # an element with index 0
        block = self.link[0]

        title = block.value["title"]
        if block.block_type == "external_link":
            return title

        # Title is optional for internal_link
        # so fallback to page's title, if it's empty
        return title or block.value["page"].title

    def get_link_url(self):
        # Link is required, so we should always have
        # an element with index 0
        block = self.link[0]

        if block.block_type == "external_link":
            return block.value["url"]

        return block.value["page"].get_url()

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = "Thank you CTA snippet"



from typing import ClassVar, List, Optional, Type

from wagtail.models import Page

from rest_framework import generics, permissions, serializers
from rest_framework.reverse import reverse

from wagtaildonate.api.metadata import HiddenMetadata
from wagtaildonate.api.serializers.utils import get_configuration_serializer_class
from wagtaildonate.configuration import get_configuration_class
from wagtaildonate.models import AbstractDonationFundraisingPayInPage
from wagtaildonate.payment_methods.base import PaymentMethod


class BaseConfigurationAPIView(generics.RetrieveAPIView):
    permission_classes = [permissions.AllowAny]
    authentication_classes = []
    checkout_url_pattern: ClassVar[Optional[str]] = "wagtaildonate:api:donations-create"
    allowed_frequencies: ClassVar[Optional[List[str]]] = None

    # Hide data on the OPTIONS request.
    metadata_class = HiddenMetadata

    def get_serializer_class(self) -> Type[serializers.Serializer]:
        return get_configuration_serializer_class()

    def get_allowed_frequencies(self) -> List[str]:
        return self.allowed_frequencies or []

    def get_checkout_url(self) -> str:
        try:
            current_app = self.request.current_app
        except AttributeError:
            try:
                current_app = self.request.resolver_match.namespace
            except AttributeError:
                current_app = None
        return self.request.build_absolute_uri(
            reverse(
                self.checkout_url_pattern,
                current_app=current_app,
                request=self.request,
            )
        )

    def get_pay_in_page(self) -> Page:
        return (
            Page.objects.live()
            .public()
            .specific()
            .filter(Page.objects.type_q(AbstractDonationFundraisingPayInPage))
            .first()
        )

    def get_pay_in_success_url(self, pay_in_page: Page) -> str:
        return pay_in_page.get_thank_you_page_url(self.request)

    def get_object_kwargs(self, **kwargs):
        kwargs.setdefault("checkout_url", self.get_checkout_url())
        kwargs.setdefault("allowed_frequencies", self.get_allowed_frequencies())
        pay_in_page = self.get_pay_in_page()
        if pay_in_page:
            kwargs.setdefault("pay_in_page_id", pay_in_page.pk)
            kwargs.setdefault(
                "pay_in_success_url", self.get_pay_in_success_url(pay_in_page)
            )
        else:
            kwargs.setdefault("pay_in_page_id", 0)
            kwargs.setdefault("pay_in_success_url", "")
        return kwargs

    def get_object(self):
        klass = get_configuration_class()
        return klass(**self.get_object_kwargs())


class DonationsConfigurationAPIView(BaseConfigurationAPIView):
    allowed_frequencies = [
        PaymentMethod.FREQUENCY_SINGLE,
        PaymentMethod.FREQUENCY_MONTHLY,
        PaymentMethod.FREQUENCY_PAYIN,
    ]

    def get_object_kwargs(self, **kwargs):
        return super().get_object_kwargs(**kwargs)
